-- Copyright (C) 2010 Tyson Brown
-- 
-- This file is part QuestHelper2, an AddOn for World of Warcraft.
-- 
-- QuestHelper2 is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
-- 
-- QuestHelper2 is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with QuestHelper2. If not, see <http://www.gnu.org/licenses/>.

if LibDebug then LibDebug() end
local shared = select(2, ...)

-- This file sets up the static map linkages that should always be available.

-- Helper function for creating a link from a single command.
local function makeLink(...)
  local link = shared.createMapLink()
  
  local args = select("#", ...)
  if args == 14 then
    local src_area, src_level, src_x, src_y, src_ex, src_ey,
          dest_area, dest_level, dest_x, dest_y, dest_ex, dest_ey, fwd_time, rev_time = ...
    link.forward_time = fwd_time
    link.reverse_time = rev_time
    link:setOrigin(src_area, src_level, src_x, src_y, src_ex, src_ey)
    link:setDestination(dest_area, dest_level, dest_x, dest_y, dest_ex, dest_ey)
  elseif args == 12 or args == 13 then
    local src_area, src_level, src_x, src_y, src_ex, src_ey,
          dest_area, dest_level, dest_x, dest_y, dest_ex, dest_ey, travel_time = ...
    link.forward_time = travel_time or 150
    link.reverse_time = travel_time or 150
    link:setOrigin(src_area, src_level, src_x, src_y, src_ex, src_ey)
    link:setDestination(dest_area, dest_level, dest_x, dest_y, dest_ex, dest_ey)
  --elseif args == 8 or args == 9 then
  elseif args == 6 or args == 7 then
    local src_area, src_x, src_y,
          dest_area, dest_x, dest_y, travel_time = ...
    link.forward_time = travel_time or 150
    link.reverse_time = travel_time or 150
    link:setOrigin(src_area, 0, src_x, src_y)
    link:setDestination(dest_area, 0, dest_x, dest_y)
  else
    error("Bad number of arguments.")
  end
end

--[[ TODO:
  
  Blizzard created maps for these zones, but I haven't yet gotten around to creating links for them:
  
  803 - The Nexus (Legendary)
  
  Additionally, Horde links to Hyjal, Twilight, Uldmn, etc. are missing or wrong. :(
]]

-- ########################################
-- Links shared by both Alliance and Horde:
-- ########################################

makeLink(464, .2344, .5353, -- Azuremyst Isle <--> Rut'theran Village
         41,  .5236, .8943)

makeLink(381, .3687, .5040, -- Darnassus <--> Rut'theran Village
         41,  .5509, .8846, 1)

makeLink(19, .5499, .5413, -- The Dark Portal <--> The Stair of Destiny
         465, .8978, .5016)

makeLink(491, .2348, .5775, -- Kamagua <--> Moa'ki Harbor
         488, .4963, .7842)

makeLink(488, .4804, .7868, -- Moa'ki Harbor <--> Unu'pe
         486, .7890, .5362)

makeLink(673, .3925, .6717, -- Booty Bay <--> Ratchet
         11, .7017, .7329)

makeLink(737, 0, .3246, .5175, nil, nil, -- The Maelstrom --> Temple of Earth
         640, 0, .4992, .5473, nil, nil, 100, nil)

makeLink(39, 0, .3825, .7749, .4255, .7184, -- Westfall <--> Deadmines
         756, 1, .2647, .1349, .7757, .3643, 1, 1)

makeLink(756, 2, .9714, .5252, .8716, .4519, -- Deadmines --> Westfall
         39, 0, .4126, .8314, .3923, .8429, 1, nil)

makeLink(301, 0, .5042, .6633, .5234, .6992, -- Stormwind City <--> The Stockade
         690, 1, .5003, .6854, .5003, .6854, 1, 1)

makeLink(21, 0, .4476, .6780, nil, nil, -- Silverpine Forest <--> Shadowfang Keep
         764, 1, .6946, .6097, .6946, .6097, 1, 1)

makeLink(22, 0, .6903, .7288, .6972, .7337, -- Western Plaguelands <--> Scholomance
         763, 1, .3908, .5947, .3908, .5947, 1, 1)

makeLink(23, 0, .2638, .1157, nil, nil, -- Eastern Plaguelands <--> Stratholme
         765, 1, .6365, .8853, .6365, .8853, 1, 1)

makeLink(23, 0, .2767, .1158, nil, nil, -- Eastern Plaguelands <--> Stratholme
         765, 1, .6900, .8859, .6900, .8859, 1, 1)

makeLink(23, 0, .4381, .1744, .4345, .1942, -- Eastern Plaguelands <--> Stratholme
         765, 2, .6737, .8654, .6737, .8654, 1, 1)

makeLink(17, 0, .3428, .1020, .4153, .1156, -- Badlands <--> Uldamman
         692, 1, .6735, .7270, .6735, .7270, 1, 1)

makeLink(27, 0, .2527, .3692, .3117, .3802, -- Dun Morogh <--> Gnomeregan
         691, 1, .6447, .2840, .6447, .2840, 1, 1)

makeLink(27, 0, .2696, .2981, .3117, .3802, -- Dun Morogh <--> Gnomeregan
         691, 3, .8516, .5911, .8516, .5911, 1, 1)

makeLink(43, 0, .1651, .1100, .1447, .1414, -- Ashnaenvale <--> Blackfathom Deeps
         688, 1, .4438, .1057, .4438, .1057, 1, 1)

makeLink(11, 0, .4204, .6663, .3893, .6912, -- Northern Barrens <--> The Wailing Caverns
         749, 1, .4557, .6085, .4557, .6085, 1, 1)

makeLink(101, 0, .3028, .5453, .2887, .6257, -- Desolace <--> Maraudon
         750, 1, .7913, .6847, .7913, .6847, 1, 1)

makeLink(101, 0, .3601, .6437, .2887, .6257, -- Desolace <--> Maraudon
         750, 1, .6230, .2919, .6230, .2919, 1, 1)

makeLink(101, 0, .2925, .6117, .2887, .6257, -- Desolace --> Maraudon
         750, 2, .2815, .3550, .2815, .3550, 1, nil)

makeLink(121, 0, .6979, .3485, .6550, .3523, -- Feralas <--> Dire Maul
         699, 5, .2763, .8524, .2763, .8524, 1, 1)

makeLink(121, 0, .6486, .3020, .6467, .3069, -- Feralas <--> Dire Maul
         699, 5, .0671, .3829, .0671, .3829, 1, 1)

--[[ This link is broken, it requires a non-existent key.
makeLink(121, 0, .7707, .3688, ?, ?, -- Feralas <--> Dire Maul
         699, 5, .9218, .4587, 9218, .4587, 1, 1) ]]

--[[ There is another exit from the Warpwood Quarter, but seems to be unreachable.
...
 ]]

makeLink(121, 0, .6030, .3129, .6048, .3078, -- Feralas <--> Dire Maul
         699, 2, .9340, .7429, .9340, .7429, 1, 1)

makeLink(121, 0, .6033, .3010, .6048, .3078, -- Feralas <--> Dire Maul
         699, 2, .9347, .4903, .9347, .4903, 1, 1)

makeLink(121, 0, .6254, .2491, .6216, .2513, -- Feralas <--> Dire Maul
         699, 1, .7065, .9348, .7065, .9348, 1, 1)

makeLink(161, 0, .3922, .2126, nil, nil, -- Tanaris <--> Zul'Farrak
         686, 0, .5660, .9095, nil, nil, 1, 1)

makeLink(504, 1, .2541, .5156, nil, nil, -- Dalaran --> Caverns of Time
         161, 0, .6525, .4979, .6497, .4999, 1, nil)

makeLink(161, 0, .6101, .6210, .6497, .4999, -- Caverns of Time <--> The Culling of Stratholme
         521, 1, .8708, .7131, .8403, .5941, 1, 1)

makeLink(161, 0, .5539, .5335, .6497, .4999, -- Caverns of Time <--> Old Hillsbrad
         734, 0, .2397, .2347, .2799, .4727, 1, 1)

makeLink(161, 0, .5690, .6233, .6497, .4999, -- Caverns of Time <--> The Black Morass
         733, 0, .5322, .0001, .5042, .0310, 1, 1)

makeLink(161, 0, .5694, .4994, .6497, .4999, -- Caverns of Time --> Battle for Mount Hyjal
           775, 0, .1068, .6474, nil, nil, 1, nil)

makeLink(38, 0, .7605, .4527, .6966, .5357, -- Swamp of Sorrows <--> Temple of Atal'Hakkar
         687, 1, .4995, .1374, .4995, .1374, 1, 1)

makeLink(20, 0, .8486, .3053, .8254, .3329, -- Trisfal Glades <--> Scarlet Monastery
         762, 1, .8392, .8296, .8392, .8296, 1, 1)

makeLink(20, 0, .8535, .3052, .8254, .3329, -- Trisfal Glades <--> Scarlet Monastery
         762, 4, .6092, .9131, .6092, .9131, 1, 1)

makeLink(20, 0, .8565, .3170, .8254, .3329, -- Trisfal Glades <--> Scarlet Monastery
         762, 3, .5986, .9817, .5986, .9817, 1, 1)

makeLink(20, 0, .8530, .3226, .8254, .3329, -- Trisfal Glades <--> Scarlet Monastery
         762, 2, .1307, .2515, .1307, .2515, 1, 1)

makeLink(607, 0, .4069, .9436, nil, nil, -- Southern Barrens <--> Razorfen Kraul
         761, 1, .7050, .8346, .7050, .8346, 1, 1)

makeLink(61, 0, .4768, .2370, nil, nil, -- Thousand Needles <--> Razorfen Downs
         760, 1, .2322, .1896, .2322, .1896, 1, 1)

makeLink(321, 2, .7019, .4899, nil, nil, -- Orgrimmar <--> Ragefire Chasm
         680, 1, .6209, .0788, .6209, .0788, 1, 1)

makeLink(28, 0, .2702, .7256, .3471, .8584, -- Searing Gorge <--> Blackrock Depths
         704, 1, .3445, .7780, .3445, .7780, 1, 1)

makeLink(28, 0, .4078, .9571, .3471, .8584, -- Searing Gorge <--> Blackrock Spire
         721, 4, .2640, .3835, .2640, .3835, 1, 1)

makeLink(28, 0, .4517, .9886, .3471, .8584, -- Searing Gorge <--> Blackrock Caverns
         753, 1, .3119, .6951, .3119, .6951, 1, 1)

makeLink(486, 0, .2750, .2592, .2781, .2832, -- Borean Tundra <--> The Nexus
         520, 1, .3715, .8885, .3715, .8885, 1, 1)

makeLink(486, 0, .2735, .2673, .2787, .2829, -- Borean Tundra <--> The Oculus
         528, 1, .6067, .4958, nil, nil, 1, 1)

makeLink(486, 0, .2751, .2681, .2707, .2566, -- Borean Tundra <--> The Eye of Eternity
         527, 1, .3098, .6033, nil, nil, 1, 1)

makeLink(488, 0, .2592, .5094, .2647, .4880, -- Dragonblight <--> Azjol Nerub
         533, 3, .0793, .9477, .0793, .9477, 1, 1)

makeLink(533, 1, .8867, .7731, .8867, .7731, -- Azjol Nerub --> Dragonblight
         488, 0, .2843, .4700, .2647, .4880, 1, nil)

makeLink(488, 0, .2850, .5175, .2647, .4880, -- Dragonblight <--> Ahn'kahet: The Old Kingdom
         522, 1, .9020, .7956, nil, nil, 1, 1)

makeLink(488, 0, .6134, .5260, .5966, .5160, -- Dragonblight <--> Ruby Sanctum
         609, 0, .4909, .2882, nil, nil, 1, 1)

makeLink(488, 0, .6002, .5966, .5966, .5160, -- Dragonblight <--> Obsidian Sanctum 
         531, 0, .6448, .5014, nil, nil, 1, 1)

makeLink(488, 0, .8710, .5064, .8701, .5097, -- Dragonblight <--> Naxxramas
         535, 5, .5389, .4965, .5389, .4965, 1, 1)

makeLink(491, 0, .5726, .4664, .5880, .4822, -- Howling Fjord <--> Utgarde Keep
         523, 1, .6989, .7527, .6989, .7527, 1, 1)

makeLink(491, 0, .5727, .4673, .5883, .4823, -- Howling Fjord <--> Utgarde Pinnacle
         524, 2, .4453, .1406, .4453, .1406, 1, 1)

makeLink(640, 0, .4770, .5196, nil, nil, -- Deepholm <--> The Stone Core
         768, 1, .5428, .9702, .5428, .9702, 1, 1)

makeLink(720, 0, .7681, .8459, nil, nil, -- Uldum <--> The Vortex Pinnacle
         769, 1, .5366, .1617, nil, nil, 1, 1)

makeLink(720, 0, .6909, .5304, .7162, .5217, -- Uldum <--> Halls of Origination
         759, 1, .5023, .9370, .5023, .9370, 1, 1)

makeLink(720, 0, .6056, .6435, nil, nil, -- Uldum <--> Lost City of the Tol'vir
         747, 0, .3138, .1457, .3220, .1913, 1, 1)

makeLink(720, 0, .3824, .8067, nil, nil, -- Uldum <--> Throne of the Four Winds
         773, 1, .4722, .7730, nil, nil, 1, 1)

makeLink(772, 0, .5890, .1436, nil, nil, -- Ahn'Qiraj: The Fallen Kingdom <--> Ruins of Ahn'Qiraj
         717, 0, .6075, .1152, nil, nil, 1, 1)

makeLink(772, 0, .4673, .0745, nil, nil, -- Ahn'Qiraj: The Fallen Kingdom <--> Ahn'Qiraj
         766, 2, .5229, .2734, nil, nil, 1, 1)

makeLink(496, 0, .7608, .2090, nil, nil, -- Zul'Drak <--> Gundrak
         530, 0, .5697, .2940, .5697, .2940, 1, 1) -- TODO: Stupid POS.

makeLink(496, 0, .8137, .2912, nil, nil, -- Zul'Drak <--> Gundrak
         530, 0, .4238, .2971, .4238, .2971, 1, 1) -- TODO: Stupid POS.

makeLink(496, 0, .2848, .8692, .2898, .8396, -- Zul'Drak <--> Drak'Tharon Keep
         534, 1, .2804, .8134, .2804, .8134, 1, 1) -- TODO: Stupid POS.

makeLink(495, 0, .4543, .2129, nil, nil, -- The Storm Peaks <--> Halls of Lightning
         525, 1, .0101, .5367, .0101, .5367, 1, 1)

makeLink(495, 0, .3950, .2691, nil, nil, -- The Storm Peaks <--> Halls of Stone
         526, 0, .5737, .4615, .5737, .4615, 1, 1) -- TODO: level should be 1, but game reports 0.

makeLink(495, 0, .4156, .1775, nil, nil, -- The Storm Peaks <--> Uldar
         529, 1, .5270, .9555, .5269, .9387, 1, 1)

makeLink(492, 0, .5481, .9183, .5267, .8938, -- Icecrown <--> Pit of Saron
         602, 0, .4088, .8125, nil, nil, 1, 1)

makeLink(492, 0, .5499, .8968, .5267, .8938, -- Icecrown <--> The Forge of Souls
         601, 1, .6668, .8848, .6668, .8848, 1, 1)

makeLink(492, 0, .5548, .9088, .5267, .8938, -- Icecrown <--> Halls Of Reflection
         603, 1, .4839, .8232, .4839, .8232, 1, 1)

makeLink(700, 0, .1913, .5377, nil, nil, -- Twilight Highlands <--> Grim Batol
         757, 1, .1057, .5632, .1057, .5632, 1, 1)

makeLink(492, 0, .5388, .8732, .5329, .8546, -- Icecrown <--> Icecrown Citadel
         604, 1, .3896, .0609, .3896, .0609, 1, 1)

makeLink(501, 0, 0.5008, .1116, .5040, .1587, -- Wintergrasp <--> Vault of Archavon
         532, 1, .4931, .8709, .4931, .8709, 1, 1)

makeLink(141, 0, .5295, .7772, .5267, .7695, -- Dustwallow Marsh <--> Onyxia's Lair
         718, 1, .3207, .2072, .3207, .2072, 1, 1)

makeLink(700, 0, .3388, .7812, nil, nil, -- Twilight Highlands <--> The Bastion of Twilight
         758, 0, .4993, .5187, .4993, .5187, 1, 1) -- TODO: level should be 1, but game reports 0.

makeLink(708, 0, .4767, .5295, .4669, .4790, -- Tol Barad <--> Baradin Hold
         752, 1, .4786, .9307, .4786, .9307, 1, 1)

makeLink(606, 0, .2750, .5634, nil, nil, -- Nordune Ridge <--> Malfurion's Breach
         795, 0, .5299, .8375, nil, nil, 1, 1)

makeLink(465, 0, .4752, .5361, nil, nil, -- Hellfire Citadel <--> Hellfire Ramparts
         797, 1, .5082, .7083, nil, nil, 1, 1)

makeLink(465, 0, .4750, .5203, nil, nil, -- Hellfire Citadel <--> The Shattered Halls
         710, 1, .6199, .9297, nil, nil, 1, 1)

makeLink(465, 0, .4590, .5190, nil, nil, -- Hellfire Citadel <--> The Blood Furnace
         725, 1, .4741, .9038, nil, nil, 1, 1)

makeLink(465, 0, .4756, .5305, .4664, .5276, -- Hellfire Citadel <--> Magtheridon's Lair
         779, 1, .6150, .1804, nil, nil, 1, 1)

makeLink(467, 0, .4894, .3580, .5038, .4087, -- Coilfang Reservoir <--> Slave Pens
         728, 1, .2177, .1397, nil, nil, 1, 1)

makeLink(467, 0, .5026, .3333, .5038, .4087, -- Coilfang Reservoir <--> Steam Vault
         727, 1, .1768, .3119, nil, nil, 1, 1)

makeLink(467, 0, .5431, .3437, .5038, .4087, -- Coilfang Reservoir <--> Underbog
         726, 1, .2822, .6998, nil, nil, 1, 1)

makeLink(467, 0, .5190, .3274, .5038, .4087, -- Coilfang Reservoir <--> Serpentshrine Cavern
         780, 1, .1349, .6225, nil, nil, 1, 1)

makeLink(478, 0, .3429, .6561, .3603, .6559, -- Terokkar Forest <--> Auchenai Crypts
         722, 1, .4406, .7756, nil, nil, 1, 1)

makeLink(478, 0, .3964, .5755, .3965, .6019, -- Terokkar Forest <--> Mana Tombs
         732, 1, .3353, .1499, nil, nil, 1, 1)

makeLink(478, 0, .4497, .6560, .4323, .6562, -- Terokkar Forest <--> Sethekk Halls
         723, 1, .7328, .3924, nil, nil, 1, 1)

makeLink(478, 0, .3963, .7364, .3964, .7098, -- Terokkar Forest <--> Shadow Labyrinth
         724, 1, .2202, .0983, nil, nil, 1, 1)

makeLink(479, 0, .7178, .5491, nil, nil, -- Tempest Keep <--> The Botanica
         729, 1, .9064, .3957, nil, nil, 1, 1)

makeLink(479, 0, .7068, .6986, nil, nil, -- Tempest Keep <--> The Mechanar
         730, 1, .4947, .8630, nil, nil, 1, 1)

makeLink(479, 0, .7450, .5767, nil, nil, -- Tempest Keep <--> The Arcatraz
         731, 1, .4126, .8456, nil, nil, 1, 1)

makeLink(479, 0, .7379, .6375, nil, nil, -- Tempest Keep <--> The Eye
         782, 1, .5004, .9185, nil, nil, 1, 1)

makeLink(475, 0, .6935, .2360, .6868, .2434, -- Blade's Edge Mountains <--> Gruul's Lair
         776, 1, .8457, .7870, nil, nil, 1, 1)

makeLink(473, 0, .7102, .4667, .7103, .4656, -- Shadowmoon Valley <--> Black Temple
         796, 2, .2182, .6131, nil, nil, 1, 1)

makeLink(499, 0, .4425, .4522, nil, nil, -- Isle of Quel'Danas <--> Sunwell Plateau
         789, 1, .3091, .3593, nil, nil, 1, 1)

makeLink(499, 0, .6130, .3092, nil, nil, -- Isle of Quel'Danas <--> Magisters' Terrace
         798, 2, .4253, .9333, nil, nil, 1, 1)

makeLink(32, 0, .4681, .7458, nil, nil, -- Deadwind Pass <--> Karazhan
         799, 1, .5994, .7846, nil, nil, 1, 1)

makeLink(32, 0, .4660, .7036, .4666, .7028, -- Deadwind Pass <--> Karazhan
         799, 6, .6460, .6077, nil, nil, 1, 1)

makeLink(463, 0, .8238, .6434, nil, nil, -- Ghostlands <--> Zul'Aman
         781, 0, .0565, .5296, .1805, .5296, 1, 1)

makeLink(37, 0, .7221, .3291, nil, nil, -- Stranglethorn <--> Zul'Gurub
         793, 0, .2871, .4885, nil, nil, 1, 1)

makeLink(481, 0, .4861, .4205, .5811, .4163, -- Terrace of Light --> Isle of Quel'Danas
         499, 0, .4825, .3448, nil, nil, 1, nil)

makeLink(606, 0, .4721, .7817, nil, nil, -- Sulfuron Spire <--> Firelands
         800, 1, .2496, .9289, nil, nil, 1, 1)

if UnitFactionGroup("player") == "Alliance" then
  -- ####################
  -- Alliance only links:
  -- ####################
  
  if UnitClass("player") == "Druid" then
    -- Alliance Druids
    makeLink(241, 0, .4425, .4531, nil, nil, -- Teldrassil --> Rut'theran Village
             41, 0, .5539, .8845, nil, nil, 150, nil)
  end
  
  makeLink(301, .2255, .5610, -- Stormwind Harbor <--> Rut'theran Village
           41,  .5502, .9370)
  
  makeLink(301, .1816, .2546, -- Stormwind Harbor <--> Valiance Keep
           486, .5970, .6942)
  
  makeLink(40, .0512, .5581, -- Baradin Bay <--> Daggercap Bay
           491, .6131, .6261)
  
  makeLink(141, .7152, .5632, -- Theramore Isle <--> Baradin Bay
           40, .0637, .6227)
  
  makeLink(341, 0, .7678, .5121, .1574, .8498, -- Tinker Town <--> Dwarven District
           301, 0, .6937, .3138, .6714, .3415)
  
  makeLink(381, 0, .4415, .7870, nil, nil, -- Temple of the Moon --> The Vault of Lights
           471, 0, .4762, .5982, nil, nil, 1, nil)
  
  makeLink(471, 0, .4757, .6214, nil, nil, -- The Vault of Lights --> Temple of the Moon
           381, 0, .4347, .7867, nil, nil, 1, nil)
  
  makeLink(381, 0, .4400, .7824, nil, nil, -- Temple of the Moon --> The Dark Portal
           19, 0, .5390, .4608, nil, nil, 1, nil)
  
  makeLink(471, 0, .4818, .6296, nil, nil, -- The Vault of Light --> The Dark Portal
           19, 0, .5390, .4608, nil, nil, 1, nil)
  
  makeLink(301, 0, .4903, .8729, .4936, .8689, -- Wizard's Sanctum --> The Dark Portal
           19, 0, .5390, .4608, nil, nil, 1, nil)
  
  makeLink(341, 0, .2715, .0717, .1574, .8498, -- Hall of Mysteries --> The Dark Portal
           19, 0, .5390, .4608, nil, nil, 1, nil)
  
  makeLink(301, 0, 0.7323, .1960, nil, nil, -- Stormwind --> Temple of Earth
           640, 0, .4873, .5356, .4992, .5482, 1, nil)
  
  makeLink(640, 0, .4855, .5382, .4992, .5482, -- Temple of Earth --> Stormwind
           301, 0, .7746, .1834, nil, nil, 1, nil)
  
  makeLink(301, 0, .7320, .1838, nil, nil, -- Stormwind --> Baradin Base Camp
           709, 0, .7367, .6092, nil, nil, 1, nil)
  
  makeLink(709, 0, .7523, .5886, nil, nil, -- Baradin Base Camp --> Stormwind
           301, 0, .7339, .1828, nil, nil, 1, nil)
  
  makeLink(301, 0, .7535, .1643, nil, nil, -- Stormwind <--> Highbank
           700, 0, .7948, .7779, nil, nil, 1, 1)
  
  makeLink(301, 0, .7619, .1868, nil, nil, -- Stormwind --> Nordrassil
           606, 0, .6349, .2337, nil, nil, 1, nil)
  
  makeLink(606, 0, .6261, .2313, nil, nil, -- Nordrassil --> Stormwind
           301, 0, .7446, .1834, nil, nil, 1, nil)
  
  makeLink(301, 0, .7525, .2051, nil, nil, -- The Eastern Earthshrine --> Ramkahen
           720, 0, .5489, .3425, nil, nil, 1, nil)
  
  -- TODO: This link goes somewhere else depending on quest progress.
  makeLink(301, 0, .7329, .1685, nil, nil, -- Stormwind --> The Briny Cutter
           610, 0, 0.4513, .2333, .4512, .2323, 1, nil)
  
  makeLink(504, 1, .4010, .6279, nil, nil, -- Dalaran --> Wizard's Sanctum
           301, 0, .4961, .8651, .4936, .8689, 1, nil)
  
  makeLink(481, 0, .5721, .4828, .5811, .4163, -- Terrace of Light --> Wizard's Sanctum
           301, 0, .4961, .8651, .4936, .8689, 1, nil)
else
  -- ####################
  -- Horde only links:
  -- ####################
  
  if UnitClass("player") == "Druid" then
    -- Horde Druid
    makeLink(241, 0, .4432, .4570, nil, nil, -- Nighthaven --> Thunderbluff
             362, 0, .4688, .4966, .4543, .5321, 150, nil)
  end
  
  makeLink(486, 0, .4138, .5361, .4267, .5571, -- Warsong Hold <--> Orgrimmar
           321, 1, .4479, .6230, nil, nil)
  
  makeLink(321, 1, .4299, .6496, nil, nil, -- Orgrimmar <--> Thunderbluff
           362, 0, .1520, .2576, nil, nil)
  
  makeLink(321, 1, .5252, .5314, nil, nil, -- Orgrimmar <--> Grom'gol Base Camp
           37, 0, .3714, .5254, nil, nil)
  
  makeLink(321, 1, .5081, .3637, nil, nil, -- Orgrimmar --> Temple of Earth
           640, 0, .4873, .5356, .4992, .5482, 1, nil)
  
  makeLink(640, 0, .5092, .5309, .4992, .5482, -- Temple of Earth --> Orgrimmar
           321, 1, .5004, .3773, nil, nil, 1, nil)
  
  makeLink(37, .3745, .5088, -- Grom'gol Base Camp <--> Trisfal Glades
           20, .6185, .5907)
  
  makeLink(20, 0, .6071, .5877, nil, nil, -- Trisfal Glades <--> Orgrimmar
           321, 1, .5079, .5587, nil, nil)
  
  makeLink(20, .5907, .5901, -- Trisfal Glades <--> Vengeance Landing
           491, .7770, .2826)
  
  makeLink(382, 0, .5485, .1121, nil, nil, -- Ruins of Lodaron --> Inner Sanctum
           480, 0, .5062, .1645, .5752, .2458, 1, nil)
  
  makeLink(480, 0, .5752, .2458, .5752, .2458, -- Inner Sanctum --> Ruins of Lodaron
           382, 0, .5693, .1140, nil, nil, 1, nil)
  
  makeLink(362, 0, .2317, .1349, .2954, .2988, -- The Pools of Vision --> The Dark Portal
           19, 0, .5390, .4608, nil, nil, 1, nil)
  
  makeLink(321, 2, .4711, .6184, .3825, .9192, -- Orgrimmar --> The Dark Portal
           19, 0, .5390, .4608, nil, nil, 1, nil)
  
  makeLink(321, 1, .3552, .6911, nil, nil, -- Orgrimmar --> The Dark Portal
           19, 0, .5390, .4608, nil, nil, 1, nil)
  
  makeLink(480, 0, .5841, .2093, .5752, .2458, -- Sunfury Spire --> The Dark Portal
           19, 0, .5390, .4608, nil, nil, 1, nil)
  
  makeLink(382, 0, .8531, .1702, .6604, .3272, -- Undercity --> The Dark Portal
           19, 0, .5390, .4608, nil, nil, 1, nil)
  
  makeLink(504, 1, .5540, .2556, nil, nil, -- Dalaran --> Orgrimmar (PROBABLY WRONG)
           321, 2, .4711, .6184, .3825, .9192, 1, nil)
  
  makeLink(481, 0, .5721, .4828, .5811, .4163, -- Terrace of Light --> Orgrimmar (PROBABLY WRONG)
           321, 2, .4711, .6184, .3825, .9192, 1, nil)
end
