local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local AddonName, a = ...
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")
local L = a.Localize
local s = SpellFlashAddon
local x = SpellFlashAddon.UpdatedVariables

local function SetColor(UseColor, Color, ElseColor)
	if UseColor then
		return Color
	end
	return ElseColor or "Pink"
end

--[[
	White - Default
	Yellow - Limited Time or No Global Cooldown
	Purple - AOE or Positional Damage
	Blue - AOE Debuff
	Orange - Finishing Move
	Aqua - Spell Interrupt, Reflect or Dispel
	Green - Self Buff or Turn Autocast On
	Red - Emergency Mitigation Cooldowns or Turn Autocast Off
	Pink - Optional
]]

-- SpellFlash API: http://wow.curseforge.com/addons/spellflash/pages/api/
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff


local whileResting, whileCombat, whileEle, whileEnhan, whileResto, ufCheck, ueCheck
local function flashCheck()
	whileResting = s.GetModuleConfig(AddonName, "Resting") or not IsResting()
	whileCombat = not s.GetModuleConfig(AddonName, "Combat") or s.InCombat()
	whileEle = not s.GetModuleConfig(AddonName, "DisableEle")
	whileEnhan = not s.GetModuleConfig(AddonName, "DisableEnhan")
	whileResto = not s.GetModuleConfig(AddonName, "DisableResto")
	whileParty = not s.GetModuleConfig(AddonName, "InParty")
end

local function Buffs()

	if s.TalentMastery(1) and whileEle and whileResting and whileCombat and not whileParty then
		
		if s.Flashable(324--[[Lightning Shield]]) and x.ActiveEnemy and a.Castable["Lightning Shield"]() then
			s.Flash(324--[[Lightning Shield]], "Green")
		end
		
		if s.Flashable(8024--[[Flametongue Weapon]]) and not s.MainHandItemBuff(L["Flametongue"]) and a.Castable["Flametongue Weapon"]() then
			s.Flash(8024--[[Flametongue Weapon]], "Green")
		end
		
	end

	if s.TalentMastery(2) and whileEnhan and whileResting and whileCombat and not whileParty then
		
		if s.Flashable(324--[[Lightning Shield]]) and x.ActiveEnemy and a.Castable["Lightning Shield"]() then
			s.Flash(324--[[Lightning Shield]], "Green")
		end
		
		if s.Flashable(8232--[[Windfury Weapon]]) and not s.MainHandItemBuff(78147--[[Windfury]], 120) and a.Castable["Windfury Weapon"]() then
			s.Flash(8232--[[Windfury Weapon]], "Green")
		end
		
		if s.Flashable(8024--[[Flametongue Weapon]]) and not s.OffHandItemBuff(L["Flametongue"], 120) and a.Castable["Flametongue Weapon"]() then
			s.Flash(8024--[[Flametongue Weapon]], "Green")
		end
			
	end
	
	if s.TalentMastery(3) and whileResto and whileResting and whileCombat and not whileParty then
		
		if s.Flashable(52127--[[Water Shield]]) and a.Castable["Water Shield"]() then
			s.Flash(52127--[[Water Shield]], "Green")
		end
			
		if s.Flashable(974--[[Earth Shield]]) and (s.MyBuffStack(974--[[Earth Shield]]) <= 1) and a.Castable["Earth Shield"]() then
			s.Flash(974--[[Earth Shield]], "Green")
		end

		if s.Flashable(51730--[[Earthliving Weapon]]) and not s.MainHandItemBuff(L["Earthliving"]) and a.Castable["Earthliving Weapon"]() then
			s.Flash(51730--[[Earthliving Weapon]], "Green")
		end
		
	end
	
end

local function Cooldowns ()

	if s.TalentMastery(1) and whileEle and whileResting and whileCombat and not whileParty then
		
		if s.Flashable(16166--[[Elemental Mastery]]) and x.ActiveEnemy and a.Castable["Elemental Mastery"]() then
			s.Flash(16166--[[Elemental Mastery]], SetColor(x.ActiveEnemy, "Orange"))
		end
		
		if s.Flashable(51490--[[Thunderstorm]]) and x.ActiveEnemy and s.PowerPercent("player") <= 70 and a.Castable["Thunderstorm"]() then
			s.Flash(51490--[[Thunderstorm]], SetColor(x.ActiveEnemy, "Red"))
		end
		
		if s.Flashable(2825--[[Bloodlust]]) and s.InCombat() and x.ActiveEnemy and a.Castable["Bloodlust"]() then
			s.Flash(2825--[[Bloodlust]], SetColor(x.ActiveEnemy, "Orange"))
		end
		
		if s.Flashable(32182--[[Heroism]]) and s.InCombat() and x.ActiveEnemy and a.Castable["Heroism"]() then		
			s.Flash(32182--[[Heroism]], SetColor(x.ActiveEnemy, "Orange"))
		end
	
	end

	if s.TalentMastery(2) and whileEnhan and whileResting and whileCombat and not whileParty then
	
		local timeLeft_UnleashElements, duration = s.SpellCooldown(73680--[[Unleash Elements]])
		local unleashFlame_Active = s.Buff(73683--[[Unleash Flame]], "player")
		local fsDuration = s.DebuffDuration(L["Flame Shock"], "target")
		local ufRemaining = s.MyBuffDuration(73683--[[Unleash Flame]], "player")
	
		if s.Flashable(51533--[[Feral Spirit]]) and x.ActiveEnemy and a.Castable["Feral Spirit"]() then
			s.Flash(51533--[[Feral Spirit]], SetColor(x.ActiveEnemy, "Orange"))
		end
	
		if s.Flashable(30823--[[Shamanistic Rage]]) and x.ActiveEnemy and s.PowerPercent("player") < 20 and s.HealthPercent("player") < 20 and a.Castable["Shamanistic Rage"]() then
			s.Flash(30823--[[Shamanistic Rage]], SetColor(x.ActiveEnemy, "Red"))
		end
	
		if s.Flashable(2825--[[Bloodlust]]) and s.InCombat() and x.ActiveEnemy and a.Castable["Bloodlust"]() then
			s.Flash(2825--[[Bloodlust]], SetColor(x.ActiveEnemy, "Orange"))
		end
		
		if s.Flashable(32182--[[Heroism]]) and s.InCombat() and x.ActiveEnemy and a.Castable["Heroism"]() then		
			s.Flash(32182--[[Heroism]], SetColor(x.ActiveEnemy, "Orange"))
		end
		
		if s.Flashable(2894--[[Fire Elemental Totem]]) and s.InCombat() and x.ActiveEnemy and a.Castable["Fire Elemental Totem"]() then
			s.Flash(2894--[[Fire Elemental Totem]], SetColor(x.ActiveEnemy, "Orange"))
		end
			
		if (fsDuration <= 4) and (timeLeft_UnleashElements >= 3) and not unleashFlame_Active and x.NoCC and a.Castable["Flame Shock"]() then
			ueCheck = true;
		else
			ueCheck = false;
		end
		
		if s.Buff(73683--[[Unleash Flame]], "player") and (ufRemaining >= 1) and (ufRemaining <= 4) then
			ufCheck = true;
		else
			ufCheck = false;
		end
	
	end
	
end

local hasST, hasWA, hasFET, hasMT, hasRT
local function Totems()

	if whileResting and whileCombat and not whileParty then
	
		-- GetTotemInfo returns: haveTotem, totemName, startTime, duration
		-- Arguments: index of the totem (Fire = 1, Earth = 2, Water = 3, Air = 4)
		local _, fireName = GetTotemInfo(1)
		local _, earthName = GetTotemInfo(2)
		local _, waterName = GetTotemInfo(3)
		local _, airName = GetTotemInfo(4)
		
		if s.TalentMastery(1) and whileEle then
			hasST = fireName == s.SpellName(3599--[[Searing Totem]], 1)
			hasWA = airName == s.SpellName(3738--[[Wrath of Air Totem]], 1)
			hasFET = fireName == s.SpellName(2894--[[Fire Elemental Totem]], 1)
			
			if s.Flashable(3599--[[Searing Totem]]) and not (hasST or hasFET) and a.Castable["Searing Totem"]() then
				s.Flash(3599--[[Searing Totem]])
			end
				
			if s.Flashable(3738--[[Wrath of Air Totem]]) and not (hasWA) and a.Castable["Wrath of Air Totem"]() then
				s.Flash(3738--[[Wrath of Air Totem]])
			end
		end
		
		if s.TalentMastery(2) and whileEnhan then
			hasST = fireName == s.SpellName(3599--[[Searing Totem]], 1)
			hasFET = fireName == s.SpellName(2894--[[Fire Elemental Totem]], 1)
			hasMT = fireName == s.SpellName(8190--[[Magma Totem]], 1)
			
			if s.Flashable(3599--[[Searing Totem]]) and not (hasST or hasFET or hasMT) and a.Castable["Searing Totem"]() then
				s.Flash(3599--[[Searing Totem]])
			end
		end
		
		if s.TalentMastery(3) and whileResto then
			hasRT = waterName == s.SpellName(5394--[[Healing Stream Totem]], 1) and airName == s.SpellName(3738--[[Wrath of Air Totem]], 1) and fireName == s.SpellName(8227--[[Flametongue Totem]], 1)
		end
		
	end
	
end


-- Use this single spam function and remove the multiple spam table below, or just use the multiple spam table below and leave this function in place.
s.Spam[AddonName] = function() if s.GetModuleConfig(AddonName, "spell_flashing_off") then return end a.RunSpamTable()
	
	flashCheck()		
	Buffs()
	Cooldowns()
	Totems()
	
	-- Elemental Rotation
	if s.TalentMastery(1) and whileEle and whileResting and whileCombat and not whileParty and x.ActiveEnemy then

		if s.Flashable(8050--[[Flame Shock]]) and x.NoCC and a.Castable["Flame Shock"]() then		
			s.Flash(8050--[[Flame Shock]])
			
		elseif s.Flashable(51505--[[Lava Burst]]) and x.NoCC and a.Castable["Lava Burst"]() then
			s.Flash(51505--[[Lava Burst]])
			
		elseif s.Flashable(8042--[[Earth Shock]]) and x.NoCC and (s.BuffStack(324--[[Lightning Shield]], "player") >= 7) and a.Castable["Earth Shock"]() then
			s.Flash(8042--[[Earth Shock]], SetColor(x.ActiveEnemy, "Yellow"))
			
		elseif s.Flashable(403--[[Lightning Bolt]]) and x.NoCC and a.Castable["Lightning Bolt"]() then
			s.Flash(403--[[Lightning Bolt]])
		
		end
					
	end
	
	-- Enhancement Rotation
	if s.TalentMastery(2) and whileEnhan and whileResting and whileCombat and not whileParty and x.ActiveEnemy then
	
		if s.Flashable(17364--[[Stormstrike]]) and x.NoCC and a.Castable["Stormstrike"]() then
			s.Flash(17364--[[Stormstrike]])
		
		elseif s.Flashable(60103--[[Lava Lash]]) and x.NoCC and a.Castable["Lava Lash"]() then
			s.Flash(60103--[[Lava Lash]])
		
		-- ufCheck Conditions:
		--[1] "Unlease Flame" buff active on player
		--[2] Remaining buff duration is between 1 and 4 seconds.
		elseif ufCheck and s.Flashable(8050--[[Flame Shock]]) and x.NoCC and a.Castable["Flame Shock"]() then
			s.Flash(8050--[[Flame Shock]])
			
		elseif s.Flashable(403--[[Lightning Bolt]]) and x.NoCC and (s.BuffStack(53817--[[Maelstrom Weapon]], "player") == 5) and a.Castable["Lightning Bolt"]() then
			s.Flash(403--[[Lightning Bolt]], SetColor(x.ActiveEnemy, "Yellow"))		
			
		elseif s.Flashable(73680--[[Unleash Elements]]) and x.NoCC and a.Castable["Unleash Elements"]() then
			s.Flash(73680--[[Unleash Elements]])
			
		elseif s.Buff(73683--[[Unleash Flame]], "player") and s.Flashable(8050--[[Flame Shock]]) and x.NoCC and a.Castable["Flame Shock"]() then
			s.Flash(8050--[[Flame Shock]])
		
		-- ueCheck Conditions:
		--[1] "Flame Shock" debuff duration is less than or equal to 4 seconds
		--[2] The cooldown on "Unleash Elements" is greater than or equal to 3 seconds
		--[3] The player does not have the "Unleash Flame" buff
		--[4] The target is not currently CC'd
		--[5] "Flame Shock" can be cast
		elseif ueCheck and s.Flashable(8050--[[Flame Shock]]) then
			s.Flash(8050--[[Flame Shock]])			
			
		elseif not ueCheck and not ufCheck and s.Flashable(8042--[[Earth Shock]]) and x.NoCC and a.Castable["Earth Shock"]() then
			s.Flash(8042--[[Earth Shock]])
			
		elseif s.Flashable(1535--[[Fire Nova]]) and (hasFET or hasMT) and a.Castable["Fire Nova"]() then
			s.Flash(1535--[[Fire Nova]])
			
		end
		
	end
	
	-- Restoration Rotation
	if s.TalentMastery(3) and whileResto and whileResting and whileCombat and not whileParty then
	
		if not (hasRT) then
			s.FlashTotemCall("Purple")
			
		end		

	end
	
end