local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local AddonName, a = ...
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")
local L = a.Localize
local s = SpellFlashAddon
local x = SpellFlashAddon.UpdatedVariables
local z = {}


-- SpellFlash API: http://wow.curseforge.com/addons/spellflash/pages/api/
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff


-- Example function:
a.Castable["English_Spell_Name"] = function()
	wipe(z)
	z.SpellID = GlobalSpellID--[[English_Spell_Name]]   --this is for defining the correct name of the spell
	z.DebuffSlotNeeded = 1   --use this if you want to make sure that a debuff slot is free on the target so that you do not replace other debuffs
	z.Debuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may only be on a target once in total from everyone and has a cooldown shorter than the debuff duration
	z.MyDebuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may be on the target multiple times by many people and has a cooldown shorter than the debuff duration
	z.Buff = z.SpellID   --place buff name or table of names here if the spell has a buff that may only be on yourself once in total from everyone and has a cooldown shorter than the buff duration
	z.MyBuff = z.SpellID   --place buff name or table of names here if the spell has a buff that may be on the target multiple times by many people and has a cooldown shorter than the buff duration
	z.BuffUnit = "player"   --you will need to use this if the buff or debuff is not applied to your target
	z.Stack = 2   --use this if the buff or debuff requires more than one application
	z.EarlyRefresh = 5   --use this to indicate the buff or debuff early for a desired number of seconds
	z.CastTime = s.CastTime(z.SpellID)   --use this only if you want to replace the spell cast time used for early indication of a buff or debuff
	z.EnemyTargetNeeded = 1   --use this if the spell should have an enemy targeted
	z.TargetThatUsesManaNeeded = 1   --use this if the spell is only useful on a target that uses mana such as mana draining spells
	z.NoRangeCheck = 1   --use this to disable range detection if the spell has a limited range but the detection in this function is not working correctly for the particular spell
	z.NotIfActive = 1   --use this if the spell may be toggled on such as auto attack or on next swing spells, or to disable when casting or channeling, or if the spell has a cooldown
	z.NotWhileMoving = 1   --use this if the spell is not able to be cast while moving
	z.NoPowerCheck = 1   --use this to disable the power checking in this function
	z.EvenIfNotUsable = 1   --use this to disable the usability checking in this function and may be used if currently in the process of completing a prerequisite that will make the spell usable
	z.Unit = "target"   --this should not be used unless the spell is not to be cast on the target
	z.Interrupt = 1   --use this if the target should be casting or channeling an interruptible ability
	return s.CheckIfSpellCastable(z)
end

-- Example function:
a.ItemCastable["English_Item_Name"] = function()
	wipe(z)
	z.ItemID = ItemID--[[English_Item_Name]]   --this is for defining the correct name of the item
	z.DebuffSlotNeeded = 1   --use this if you want to make sure that a debuff slot is free on the target so that you do not replace other debuffs
	z.Debuff = GlobalSpellID--[[English_Spell_Name]]   --place debuff name or table of names here if the item has a debuff that may only be on a target once in total from everyone and has a cooldown shorter than the debuff duration
	z.MyDebuff = GlobalSpellID--[[English_Spell_Name]]   --place debuff name or table of names here if the item has a debuff that may be on the target multiple times by many people and has a cooldown shorter than the debuff duration
	z.Buff = GlobalSpellID--[[English_Spell_Name]]   --place buff name or table of names here if the item has a buff that may only be on yourself once in total from everyone and has a cooldown shorter than the buff duration
	z.MyBuff = GlobalSpellID--[[English_Spell_Name]]   --place buff name or table of names here if the item has a buff that may be on the target multiple times by many people and has a cooldown shorter than the buff duration
	z.BuffUnit = "player"   --you will need to use this if the buff or debuff is not applied to your target
	z.Stack = 2   --use this if the buff or debuff requires more than one application
	z.EarlyRefresh = 5   --use this to indicate the buff or debuff early for a desired number of seconds
	z.CastTime = 0   --use this if the item has a cast time for indication of a buff or debuff early
	z.EnemyTargetNeeded = 1   --use this if the item should have an enemy targeted
	z.TargetThatUsesManaNeeded = 1   --use this if the item is only useful on a target that uses mana such as mana draining items
	z.NoRangeCheck = 1   --use this to disable range detection if the item has a limited range but the detection in this function is not working correctly for the particular item
	z.NoEquipCheck = 1   --use this to disable equippable detection if the item can be equipped but does not need to be
	z.NotIfActive = 1   --use this if the item may be toggled on such as auto attack or on next swing spells, or to disable when casting or channeling, or if the item has a cooldown
	z.NotWhileMoving = 1   --use this if the item is not able to be used while moving
	z.NoPowerCheck = 1   --use this to disable the power checking in this function
	z.EvenIfNotUsable = 1   --use this to disable the usability checking in this function and may be used if currently in the process of completing a prerequisite that will make the item usable
	z.Unit = "target"   --this should not be used unless the item is not to be cast on the target
	z.Interrupt = 1   --use this if the target should be casting or channeling an interruptible ability
	return s.CheckIfItemCastable(z)
end

-- Example function:
a.VehicleCastable["English_Spell_Name"] = function()
	wipe(z)
	z.SpellID = GlobalSpellID--[[English_Spell_Name]]   --this is for defining the correct name of the spell
	z.DebuffSlotNeeded = 1   --use this if you want to make sure that a debuff slot is free on the target so that you do not replace other debuffs
	z.Debuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may only be on a target once in total from everyone and has a cooldown shorter than the debuff duration
	z.MyDebuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may be on the target multiple times by many people and has a cooldown shorter than the debuff duration
	z.Buff = z.SpellID   --place buff name or table of names here if the spell has a buff that may only be on yourself once in total from everyone and has a cooldown shorter than the buff duration
	z.MyBuff = z.SpellID   --place buff name or table of names here if the spell has a buff that may be on the target multiple times by many people and has a cooldown shorter than the buff duration
	z.BuffUnit = "vehicle"   --you will need to use this if the buff or debuff is not applied to your target
	z.Stack = 2   --use this if the buff or debuff requires more than one application
	z.EarlyRefresh = 5   --use this to indicate the buff or debuff early for a desired number of seconds
	z.CastTime = 0   --use this if the spell has a cast time for indication of a buff or debuff early
	z.EnemyTargetNeeded = 1   --use this if the spell should have an enemy targeted
	z.TargetThatUsesManaNeeded = 1   --use this if the spell is only useful on a target that uses mana such as mana draining spells
	z.NoRangeCheck = 1   --use this to disable range detection if the spell has a limited range but the detection in this function is not working correctly for the particular spell
	z.NotIfActive = 1   --use this if the spell may be toggled on such as auto attack or on next swing spells, or to disable when casting or channeling, or if the spell has a cooldown
	z.NotWhileMoving = 1   --use this if the spell is not able to be cast while moving
	z.NoPowerCheck = 1   --use this to disable the power checking in this function
	z.EvenIfNotUsable = 1   --use this to disable the usability checking in this function and may be used if currently in the process of completing a prerequisite that will make the spell usable
	z.GlobalVehicleCooldownSpell = GlobalSpellID--[[English_Spell_Name]]   -- use this if spell has more than a global cooldown
	z.Unit = "target"   --this should not be used unless the spell is not to be cast on the target
	z.Interrupt = 1   --use this if the target should be casting or channeling an interruptible ability
	return s.CheckIfVehicleSpellCastable(z)
end



a.Castable["Auto Attack"] = function()
	wipe(z)
	z.SpellID = 6603--[[Auto Attack]]
	z.EnemyTargetNeeded = 1
	z.NotIfActive = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

a.Castable["Shoot"] = function()
	wipe(z)
	z.SpellID = 3018--[[Shoot]]
	z.EnemyTargetNeeded = 1
	z.NotIfActive = 1
	return not s.Shooting() and s.CheckIfSpellCastable(z)
end


--------------Items--------------
a.ItemCastable["Potion of the Tol'vir"] = function()
	wipe(z)
	z.ItemID = 58145--[[Potion of the Tol'vir]]
	z.EnemyTargetNeeded = 1  
	return s.CheckIfItemCastable(z)
end

--------------Buffs------------
local MarkoftheWildBuffs = {
	1126--[[Mark of the Wild]],
	20217--[[Blessing of Kings]],
}
a.Castable["Mark of the Wild"] = function()
	wipe(z)
	z.SpellID = 1126--[[Mark of the Wild]]
	z.Buff = MarkoftheWildBuffs
	z.BuffUnit = "raid|all|range"
	z.NoRangeCheck = 1
	return x.ShouldTemporaryBuff and s.CheckIfSpellCastable(z)
end

--------------General------------
local function OtherAurasFunction()
	s.SetOtherAuras(16857--[[Faerie Fire (Feral)]], 91565--[[Faerie Fire]])
end
s.RegisterOtherAurasFunction(OtherAurasFunction)
local FaerieFireAndSunderArmor = {
	91565--[[Faerie Fire]],
	7386--[[Sunder Armor]],
}

a.Castable["Entangling Roots"] = function()
	wipe(z)
	z.SpellID = 339--[[Entangling Roots]]
	if UnitExists("focus") and not s.Boss("focus") and ( not s.Debuff(nil, "focus") or s.NoDamageCC("focus") ) then
		z.Unit = "focus"
	end
	--z.NotWhileMoving = 1
	z.EnemyTargetNeeded = 1
	local seconds = s.CastTime(z.SpellID) + s.SpellCooldown(z.SpellID) + x.DoubleLag
	return not s.Boss(z.Unit) and s.CheckIfSpellCastable(z) and ( not s.Debuff(nil, z.Unit) or ( s.NoDamageCC(z.Unit) and not s.ImmunityDebuff(z.Unit, seconds) and not s.BreakOnDamageCC(z.Unit, seconds + 5) ) )
end

a.Castable["Hibernate"] = function()
	wipe(z)
	z.SpellID = 2637--[[Hibernate]]
	if UnitExists("focus") and not s.Boss("focus") and ( not s.Debuff(nil, "focus") or s.NoDamageCC("focus") ) then
		z.Unit = "focus"
	end
	--z.NotWhileMoving = 1
	z.EnemyTargetNeeded = 1
	local seconds = s.CastTime(z.SpellID) + s.SpellCooldown(z.SpellID) + x.DoubleLag
	return not s.Boss(z.Unit) and s.CheckIfSpellCastable(z) and ( not s.Debuff(nil, z.Unit) or ( s.NoDamageCC(z.Unit) and not s.ImmunityDebuff(z.Unit, seconds) and not s.BreakOnDamageCC(z.Unit, seconds + 5) ) )
end

---------------------- Abilities ------------------------
----------Poisons/Buffs-----------

a.ItemCastable["Instant Poison"] = function()
	wipe(z)
	z.ItemID = 6947--[[Instant Poison]]
	return s.CheckIfItemCastable(z)
end
a.ItemCastable["Crippling Poison"] = function()
	wipe(z)
	z.ItemID = 3775--[[Crippling Poison]]
	return s.CheckIfItemCastable(z)
end
a.ItemCastable["Deadly Poison"] = function()
	wipe(z)
	z.ItemID = 2892--[[Deadly Poison]]
	return s.CheckIfItemCastable(z)
end
a.ItemCastable["Mind-Numbing Poison"] = function()
	wipe(z)
	z.ItemID = 5237--[[Mind-Numbing Poison]]
	return s.CheckIfItemCastable(z)
end
a.ItemCastable["Wound Poison"] = function()
	wipe(z)
	z.ItemID = 10918--[[Wound Poison]]
	return s.CheckIfItemCastable(z)
end
----------Stance--------
--Stealth - 1784
a.Castable["Stealth"] = function()
	wipe(z)
	z.SpellID = 1784--[[Stealth]]
	if s.Form(z.SpellID) then
		return false
	end
	return s.CheckIfSpellCastable(z)
end
----------Talent--------

---------------------- Subtlety ------------------------
	
--Blind - 2094
--Cloak of Shadows - 31224
--Disarm Trap - 1842
--Distract - 1725

a.Castable["Hemorrhage"] = function()
	wipe(z)
	z.SpellID = 16511--[[Hemorrhage]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

--Pick Lock - 1804
--Pick Pocket - 921
--Redirect - 73981
--Sap - 6770

a.Castable["Shadowstep"] = function()
	wipe(z)
	z.SpellID = 36554--[[Shadowstep]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

--Smoke Bomb - 76577

--Tricks of the Trade - 57934

a.Castable["Vanish"] = function()
	wipe(z)
	z.SpellID = 1856--[[Vanish]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end


---------------------- Assassination ------------------------
a.Castable["Vendetta"] = function()
	wipe(z)
	z.SpellID = 79140--[[Vendetta]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Ambush"] = function()
	wipe(z)
	z.SpellID = 8676--[[Ambush]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
--Cheap Shot - 1833

a.Castable["Cold Blood"] = function()
	wipe(z)
	z.SpellID = 14177--[[Cold Blood]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
--Deadly Throw - 26679
a.Castable["Dismantle"] = function()
	wipe(z)
	z.SpellID = 51722--[[Dismantle]]
	z.Debuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Envenom"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 32645--[[Envenom]]
	z.Buff = 5171--[[Slice and Dice]]
	z.BuffUnit = "player"
	z.EarlyRefresh = EarlyRefreshSeconds or 5
	z.NoRangeCheck = 1
	return s.CheckIfSpellCastable(z)
end


a.Castable["Eviscerate"] = function()
	wipe(z)
	z.SpellID = 2098--[[Eviscerate]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

--Expose Armor - 8647

a.Castable["Garrote"] = function()
	wipe(z)
	z.SpellID = 703--[[Garrote]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
--Kidney Shot - 408

a.Castable["Mutilate"] = function()
	wipe(z)
	z.SpellID = 1329--[[Mutilate]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

a.Castable["Rupture"] = function()
	wipe(z)
	z.SpellID = 1943--[[Rupture]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

a.Castable["Slice and Dice"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 5171--[[Slice and Dice]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	z.EarlyRefresh = EarlyRefreshSeconds or 2
	z.NoRangeCheck = 1
	return s.CheckIfSpellCastable(z)
end


---------------------- Combat ------------------------

a.Castable["Backstab"] = function()
	wipe(z)
	z.SpellID = 53--[[Backstab]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
--Blade Flurry - 13877
--Combat Readiness - 74001
--Evasion - 5277
--Fan of Knives - 51723
--Feint - 1966
--Gouge - 1776

a.Castable["Kick"] = function()
	wipe(z)
	z.SpellID = 1766--[[Kick]]
	z.MyDebuff = z.SpellID 
	z.EnemyTargetNeeded = 1
	return s.GetCastingOrChanneling(nil, nil, 1) > (s.SpellCooldown(z.SpellID) + x.DoubleLag) and s.CheckIfSpellCastable(z)
end

a.Castable["Recuperate"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 73651--[[Recuperate]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	z.EarlyRefresh = EarlyRefreshSeconds or 2
	z.NoRangeCheck = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

--Shiv - 5938
--Sinister Strike - 1752
--Sinister Strike Enabler - 79327
--Sprint - 2983



-------------END CODE-----------------



a.Castable["Barkskin"] = function()
	wipe(z)
	z.SpellID = 22812--[[Barkskin]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Bear Form"] = function()
	wipe(z)
	z.SpellID = 5487--[[Bear Form]]
	if s.Form(z.SpellID) or s.Form(768--[[Cat Form]]) or s.Form(24858--[[Moonkin Form]]) then
		return false
	end
	return s.CheckIfSpellCastable(z)
end
a.Castable["Berserk"] = function()
	wipe(z)
	z.SpellID = 50334--[[Berserk]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Cat Form"] = function()
	wipe(z)
	z.SpellID = 768--[[Cat Form]]
	if s.Form(z.SpellID) or s.Form(5487--[[Bear Form]]) or s.Form(24858--[[Moonkin Form]]) then
		return false
	end
	return s.CheckIfSpellCastable(z)
end

a.Castable["Faerie Fire (Feral)"] = function()
	wipe(z)
	z.SpellID = 16857--[[Faerie Fire (Feral)]]
	z.Debuff = FaerieFireAndSunderArmor
	z.Stack = 3
	z.EarlyRefresh = 10
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Moonkin Form"] = function()
	wipe(z)
	z.SpellID = 24858--[[Moonkin Form]]
	if s.Form(z.SpellID) or s.Form(5487--[[Bear Form]]) or s.Form(768--[[Cat Form]]) then
		return false
	end
	return s.CheckIfSpellCastable(z)
end
a.Castable["Survival Instincts"] = function()
	wipe(z)
	z.SpellID = 61336--[[Survival Instincts]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Travel Form"] = function()
	wipe(z)
	z.SpellID = 783--[[Travel Form]]
	if s.Form(z.SpellID) then
		return false
	end
	return s.CheckIfSpellCastable(z)
end


---------------------- Feral(Cat) ------------------------

a.Castable["Claw"] = function()
	wipe(z)
	z.SpellID = 1082--[[Claw]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Cower"] = function()
	wipe(z)
	z.SpellID = 8998--[[Cower]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Dash"] = function()
	wipe(z)
	z.SpellID = 1850--[[Dash]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Feral Charge(Cat Form)"] = function()
	wipe(z)
	z.SpellID = 49376--[[Feral Charge(Cat Form)]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Ferocious Bite"] = function()
	wipe(z)
	z.SpellID = 22568--[[Ferocious Bite]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Maim"] = function()
	wipe(z)
	z.SpellID = 22570--[[Maim]]
	z.MyDebuff = z.SpellID 
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Nature's Grasp"] = function()
	wipe(z)
	z.SpellID = 16689--[[Nature's Grasp]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

local MangleAndBloodFrenzy = {
	33876--[[Mangle]],
	46856--[[Trauma]],
	16511--[[Hemorrhage]],
}
a.Castable["Mangle(Cat Form)2"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 33876--[[Mangle(Cat Form)]]
	z.Debuff = MangleAndBloodFrenzy
	z.EarlyRefresh = EarlyRefreshSeconds or 5
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Mangle(Cat Form)"] = function()
	wipe(z)
	z.SpellID = 33876--[[Mangle(Cat Form)]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

a.Castable["Pounce"] = function()
	wipe(z)
	z.SpellID = 9005--[[Pounce]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Prowl"] = function()
	wipe(z)
	z.SpellID = 5215--[[Prowl]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Rake"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 1822--[[Rake]]
	z.MyDebuff = z.SpellID
	z.EarlyRefresh = EarlyRefreshSeconds or 3
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
local Ravages = {
	81170--[[Ravage!]],
	6785--[[Ravage]],
}
a.Castable["Ravage!"] = function()
	wipe(z)
	z.SpellID = 81170--[[Ravage!]] or 6785--[[Ravage]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Ravage"] = function()
	wipe(z)
	z.SpellID = 6785--[[Ravage]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Rip"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 1079--[[Rip]]
	z.MyDebuff = z.SpellID
	z.EarlyRefresh = EarlyRefreshSeconds or 3
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Savage Roar"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 52610--[[Savage Roar]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	z.EarlyRefresh = EarlyRefreshSeconds or 2
	z.NoRangeCheck = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Shred"] = function()
	wipe(z)
	z.SpellID = 5221--[[Shred]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Skull Bash(Cat Form)"] = function()
	wipe(z)
	z.SpellID = 80965--[[Skull Bash(Cat Form)]]
	z.MyDebuff = z.SpellID 
	z.EnemyTargetNeeded = 1
	return s.GetCastingOrChanneling(nil, nil, 1) > (s.SpellCooldown(z.SpellID) + x.DoubleLag) and s.CheckIfSpellCastable(z)
end
a.Castable["Stampeding Roar"] = function()
	wipe(z)
	z.SpellID = 77764--[[Stampeding Roar]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Swipe(Cat Form)"] = function()
	wipe(z)
	z.SpellID = 62078--[[Swipe(Cat Form)]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Tiger's Fury"] = function()
	wipe(z)
	z.SpellID = 5217--[[Tiger's Fury]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end


---------------------- Feral(Bear) ------------------------

a.Castable["Bash"] = function()
	wipe(z)
	z.SpellID = 5211--[[Bash]]
	z.EnemyTargetNeeded = 1
	return s.GetCastingOrChanneling(nil, nil, 1) > (s.SpellCooldown(z.SpellID) + x.DoubleLag) and s.CheckIfSpellCastable(z)
end
a.Castable["Challenging Roar"] = function()
	wipe(z)
	z.SpellID = 5209--[[Challenging Roar]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
local DemoShoutAll = {
	99--[[Demoralizing Roar]],
	1160--[[Demoralizing Shout]],
	26017--[[Vindication]],
	81131--[[Scarlet Fever]],
	702--[[Curse of Weakness]],
	24423--[[Demoralizing Screech]],
	50256--[[Demoralizing Roar]],
}
a.Castable["Demoralizing Roar"] = function()
	wipe(z)
	z.SpellID = 99--[[Demoralizing Roar]]
	z.Debuff = DemoShoutAll
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Enrage"] = function()
	wipe(z)
	z.SpellID = 5229--[[Enrage]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Feral Charge(Bear Form)"] = function()
	wipe(z)
	z.SpellID = 16979--[[Feral Charge(Bear Form)]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Frenzied Regeneration"] = function()
	wipe(z)
	z.SpellID = 22842--[[Frenzied Regeneration]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Growl"] = function()
	wipe(z)
	z.SpellID = 6795--[[Growl]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Lacerate"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 33745--[[Lacerate]]
	z.MyDebuff = z.SpellID
	z.Stack = 3
	z.EarlyRefresh = EarlyRefreshSeconds or 5
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Mangle(Bear Form)"] = function()
	wipe(z)
	z.SpellID = 33878--[[Mangle(Bear Form)]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Maul"] = function()
	wipe(z)
	z.SpellID = 6807--[[Maul]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Pulverize"] = function()
	wipe(z)
	z.SpellID = 80313--[[Pulverize]]
	z.Buff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Skull Bash(Bear Form)"] = function()
	wipe(z)
	z.SpellID = 80964--[[Skull Bash(Bear Form)]]
	z.MyDebuff = z.SpellID 
	z.EnemyTargetNeeded = 1
	return s.GetCastingOrChanneling(nil, nil, 1) > (s.SpellCooldown(z.SpellID) + x.DoubleLag) and s.CheckIfSpellCastable(z)
end
a.Castable["Stampeding Roar"] = function()
	wipe(z)
	z.SpellID = 77761--[[Stampeding Roar]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Swipe(Bear Form)"] = function()
	wipe(z)
	z.SpellID = 779--[[Swipe(Bear Form)]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end
a.Castable["Thrash"] = function()
	wipe(z)
	z.SpellID = 77758--[[Thrash]]
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

---------------------- Boomkin ------------------------

a.Castable["Cyclone"] = function()
	wipe(z)
	z.SpellID = 33786--[[Cyclone]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Faerie Fire"] = function()
	wipe(z)
	z.SpellID = 770--[[Faerie Fire]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Force of Nature"] = function()
	wipe(z)
	z.SpellID = 33831--[[Force of Nature]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Hurricane"] = function()
	wipe(z)
	z.SpellID = 16914--[[Hurricane]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Innervate"] = function()
	wipe(z)
	z.SpellID = 29166--[[Innervate]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	z.NoRangeCheck = 1
	return ( x.Enemy or not UnitExists(s.UnitSelection()) or UnitIsUnit(s.UnitSelection(), "player") ) and s.CheckIfSpellCastable(z)
end
a.Castable["Insect Swarm"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 5570--[[Insect Swarm]]
	z.MyDebuff = z.SpellID
	z.EarlyRefresh = EarlyRefreshSeconds or 3
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
local MoonSunfire = {
	8921--[[Moonfire]],
	93402--[[Sunfire]],
}
a.Castable["Moonfire"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 8921--[[Moonfire]]
	z.MyDebuff = MoonSunfire
	z.EarlyRefresh = EarlyRefreshSeconds or 3
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Solar Beam"] = function()
	wipe(z)
	z.SpellID = 78675--[[Solar Beam]]
	z.MyDebuff = z.SpellID 
	z.EnemyTargetNeeded = 1
	return s.GetCastingOrChanneling(nil, nil, 1) > (s.SpellCooldown(z.SpellID) + x.DoubleLag) and s.CheckIfSpellCastable(z)
end
a.Castable["Soothe"] = function()
	wipe(z)
	z.SpellID = 2908--[[Soothe]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Starfall"] = function()
	wipe(z)
	z.SpellID = 48505--[[Starfall]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Starfire"] = function()
	wipe(z)
	z.SpellID = 2912--[[Starfire]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Starsurge"] = function()
	wipe(z)
	z.SpellID = 78674--[[Starsurge]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Wrath"] = function()
	wipe(z)
	z.SpellID = 5176--[[Wrath]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

---------------------- Resto ------------------------


