local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local AddonName, a = ...
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")
local L = a.Localize
local s = SpellFlashAddon
local x = SpellFlashAddon.UpdatedVariables
local z = {}


-- SpellFlash API: http://wow.curseforge.com/addons/spellflash/pages/api/
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff


-- Example function:
a.Castable["English_Spell_Name"] = function()
	wipe(z)
	z.SpellID = GlobalSpellID--[[English_Spell_Name]]   --this is for defining the correct name of the spell
	z.DebuffSlotNeeded = 1   --use this if you want to make sure that a debuff slot is free on the target so that you do not replace other debuffs
	z.Debuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may only be on a target once in total from everyone and has a cooldown shorter than the debuff duration
	z.MyDebuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may be on the target multiple times by many people and has a cooldown shorter than the debuff duration
	z.Buff = z.SpellID   --place buff name or table of names here if the spell has a buff that may only be on yourself once in total from everyone and has a cooldown shorter than the buff duration
	z.MyBuff = z.SpellID   --place buff name or table of names here if the spell has a buff that may be on the target multiple times by many people and has a cooldown shorter than the buff duration
	z.BuffUnit = "player"   --you will need to use this if the buff or debuff is not applied to your target
	z.Stack = 2   --use this if the buff or debuff requires more than one application
	z.EarlyRefresh = 5   --use this to indicate the buff or debuff early for a desired number of seconds
	z.CastTime = s.CastTime(z.SpellID)   --use this only if you want to replace the spell cast time used for early indication of a buff or debuff
	z.EnemyTargetNeeded = 1   --use this if the spell should have an enemy targeted
	z.TargetThatUsesManaNeeded = 1   --use this if the spell is only useful on a target that uses mana such as mana draining spells
	z.NoRangeCheck = 1   --use this to disable range detection if the spell has a limited range but the detection in this function is not working correctly for the particular spell
	z.NotIfActive = 1   --use this if the spell may be toggled on such as auto attack or on next swing spells, or to disable when casting or channeling, or if the spell has a cooldown
	z.NotWhileMoving = 1   --use this if the spell is not able to be cast while moving
	z.NoPowerCheck = 1   --use this to disable the power checking in this function
	z.EvenIfNotUsable = 1   --use this to disable the usability checking in this function and may be used if currently in the process of completing a prerequisite that will make the spell usable
	z.Unit = "target"   --this should not be used unless the spell is not to be cast on the target
	z.Interrupt = 1   --use this if the target should be casting or channeling an interruptible ability
	return s.CheckIfSpellCastable(z)
end

-- Example function:
a.ItemCastable["English_Item_Name"] = function()
	wipe(z)
	z.ItemID = ItemID--[[English_Item_Name]]   --this is for defining the correct name of the item
	z.DebuffSlotNeeded = 1   --use this if you want to make sure that a debuff slot is free on the target so that you do not replace other debuffs
	z.Debuff = GlobalSpellID--[[English_Spell_Name]]   --place debuff name or table of names here if the item has a debuff that may only be on a target once in total from everyone and has a cooldown shorter than the debuff duration
	z.MyDebuff = GlobalSpellID--[[English_Spell_Name]]   --place debuff name or table of names here if the item has a debuff that may be on the target multiple times by many people and has a cooldown shorter than the debuff duration
	z.Buff = GlobalSpellID--[[English_Spell_Name]]   --place buff name or table of names here if the item has a buff that may only be on yourself once in total from everyone and has a cooldown shorter than the buff duration
	z.MyBuff = GlobalSpellID--[[English_Spell_Name]]   --place buff name or table of names here if the item has a buff that may be on the target multiple times by many people and has a cooldown shorter than the buff duration
	z.BuffUnit = "player"   --you will need to use this if the buff or debuff is not applied to your target
	z.Stack = 2   --use this if the buff or debuff requires more than one application
	z.EarlyRefresh = 5   --use this to indicate the buff or debuff early for a desired number of seconds
	z.CastTime = 0   --use this if the item has a cast time for indication of a buff or debuff early
	z.EnemyTargetNeeded = 1   --use this if the item should have an enemy targeted
	z.TargetThatUsesManaNeeded = 1   --use this if the item is only useful on a target that uses mana such as mana draining items
	z.NoRangeCheck = 1   --use this to disable range detection if the item has a limited range but the detection in this function is not working correctly for the particular item
	z.NoEquipCheck = 1   --use this to disable equippable detection if the item can be equipped but does not need to be
	z.NotIfActive = 1   --use this if the item may be toggled on such as auto attack or on next swing spells, or to disable when casting or channeling, or if the item has a cooldown
	z.NotWhileMoving = 1   --use this if the item is not able to be used while moving
	z.NoPowerCheck = 1   --use this to disable the power checking in this function
	z.EvenIfNotUsable = 1   --use this to disable the usability checking in this function and may be used if currently in the process of completing a prerequisite that will make the item usable
	z.Unit = "target"   --this should not be used unless the item is not to be cast on the target
	z.Interrupt = 1   --use this if the target should be casting or channeling an interruptible ability
	return s.CheckIfItemCastable(z)
end

-- Example function:
a.VehicleCastable["English_Spell_Name"] = function()
	wipe(z)
	z.SpellID = GlobalSpellID--[[English_Spell_Name]]   --this is for defining the correct name of the spell
	z.DebuffSlotNeeded = 1   --use this if you want to make sure that a debuff slot is free on the target so that you do not replace other debuffs
	z.Debuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may only be on a target once in total from everyone and has a cooldown shorter than the debuff duration
	z.MyDebuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may be on the target multiple times by many people and has a cooldown shorter than the debuff duration
	z.Buff = z.SpellID   --place buff name or table of names here if the spell has a buff that may only be on yourself once in total from everyone and has a cooldown shorter than the buff duration
	z.MyBuff = z.SpellID   --place buff name or table of names here if the spell has a buff that may be on the target multiple times by many people and has a cooldown shorter than the buff duration
	z.BuffUnit = "vehicle"   --you will need to use this if the buff or debuff is not applied to your target
	z.Stack = 2   --use this if the buff or debuff requires more than one application
	z.EarlyRefresh = 5   --use this to indicate the buff or debuff early for a desired number of seconds
	z.CastTime = 0   --use this if the spell has a cast time for indication of a buff or debuff early
	z.EnemyTargetNeeded = 1   --use this if the spell should have an enemy targeted
	z.TargetThatUsesManaNeeded = 1   --use this if the spell is only useful on a target that uses mana such as mana draining spells
	z.NoRangeCheck = 1   --use this to disable range detection if the spell has a limited range but the detection in this function is not working correctly for the particular spell
	z.NotIfActive = 1   --use this if the spell may be toggled on such as auto attack or on next swing spells, or to disable when casting or channeling, or if the spell has a cooldown
	z.NotWhileMoving = 1   --use this if the spell is not able to be cast while moving
	z.NoPowerCheck = 1   --use this to disable the power checking in this function
	z.EvenIfNotUsable = 1   --use this to disable the usability checking in this function and may be used if currently in the process of completing a prerequisite that will make the spell usable
	z.GlobalVehicleCooldownSpell = GlobalSpellID--[[English_Spell_Name]]   -- use this if spell has more than a global cooldown
	z.Unit = "target"   --this should not be used unless the spell is not to be cast on the target
	z.Interrupt = 1   --use this if the target should be casting or channeling an interruptible ability
	return s.CheckIfVehicleSpellCastable(z)
end



a.Castable["Auto Attack"] = function()
	wipe(z)
	z.SpellID = 6603--[[Auto Attack]]
	z.EnemyTargetNeeded = 1
	z.NotIfActive = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

a.Castable["Shoot"] = function()
	wipe(z)
	z.SpellID = 3018--[[Shoot]]
	z.EnemyTargetNeeded = 1
	z.NotIfActive = 1
	return not s.Shooting() and s.CheckIfSpellCastable(z)
end


a.ItemCastable["Mana Gem"] = function()
	wipe(z)
	z.ItemID = 36799--[[Mana Gem]]
	z.EnemyTargetNeeded = 1  
	return s.CheckIfItemCastable(z)
end

a.Castable["Conjure Mana Gem"] = function()
	wipe(z)
	z.SpellID = 759--[[Conjure Mana Gem]]
	z.NotIfActive = 1
	--z.NotWhileMoving = 1
	return GetItemCount(36799--[[Mana Gem]], false, true) <= 2 and s.CheckIfSpellCastable(z)
end


function a.IsArmorActive()
	return s.Buff({
		30482--[[Molten Armor]],
		7302--[[Frost Armor]],
		6117--[[Mage Armor]],
	}, "player")
end

--Mage Functions
--Buffs
a.Castable["Molten Armor"] = function()
	wipe(z)
	z.SpellID = 30482--[[Molten Armor]]
	z.Buff = {
		30482--[[Molten Armor]],
		7302--[[Frost Armor]],
		6117--[[Mage Armor]],
	}
	z.BuffUnit = "player"
	return s.CheckIfSpellCastable(z)
end

a.Castable["Frost Armor"] = function()
	wipe(z)
	z.SpellID = 7302--[[Frost Armor]]
	z.Buff = {
		30482--[[Molten Armor]],
		7302--[[Frost Armor]],
		6117--[[Mage Armor]],
	}
	z.BuffUnit = "player"
	return s.CheckIfSpellCastable(z)
end

a.Castable["Mage Armor"] = function()
	wipe(z)
	z.SpellID = 6117--[[Mage Armor]]
	z.Buff = {
		30482--[[Molten Armor]],
		7302--[[Frost Armor]],
		6117--[[Mage Armor]],
	}
	z.BuffUnit = "player"
	return s.CheckIfSpellCastable(z)
end

a.Castable["Mage Armor Mana"] = function()
	wipe(z)
	z.SpellID = 6117--[[Mage Armor]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	return s.CheckIfSpellCastable(z)
end

a.Castable["Molten Armor DPS"] = function()
	wipe(z)
	z.SpellID = 30482--[[Molten Armor]]
	z.Buff = z.SpellID	
	z.BuffUnit = "player"
	return s.CheckIfSpellCastable(z)
end

local IntelBuffs = {
	1459--[[Arcane Brilliance]],
	61316--[[Dalaran Brilliance]],
}

a.Castable["Arcane Brilliance"] = function()
	wipe(z)
	z.SpellID = 1459--[[Arcane Brilliance]]
	z.Buff = IntelBuffs
	z.BuffUnit = "raid|all|range"
	z.NoRangeCheck = 1
	return x.ShouldTemporaryBuff and s.CheckIfSpellCastable(z)
end

a.Castable["Dalaran Brilliance"] = function()
	wipe(z)
	z.SpellID = 61316--[[Dalaran Brilliance]]
	z.Buff = IntelBuffs
	z.BuffUnit = "raid|all|range"
	z.NoRangeCheck = 1
	return x.ShouldTemporaryBuff and s.CheckIfSpellCastable(z)
end

--General
local PolyEffects = {
	118--[[Polymorph]],
	61305--[[Polymorph]],
	28272--[[Polymorph]],
	61721--[[Polymorph]],
	61780--[[Polymorph]],
	28271--[[Polymorph]],
}
a.Castable["Polymorph"] = function()
	wipe(z)
	z.SpellID = 118--[[Polymorph]]
	if UnitExists("focus") and not s.Boss("focus") and ( not s.Debuff(nil, "focus") or s.NoDamageCC("focus") ) then
		z.Unit = "focus"
	end
	--z.NotWhileMoving = 1
	z.EnemyTargetNeeded = 1
	local seconds = s.CastTime(z.SpellID) + s.SpellCooldown(z.SpellID) + x.DoubleLag
	return not s.Boss(z.Unit) and s.CheckIfSpellCastable(z) and ( not s.Debuff(nil, z.Unit) or ( s.NoDamageCC(z.Unit) and not s.ImmunityDebuff(z.Unit, seconds) and not s.BreakOnDamageCC(z.Unit, seconds + 5) ) )
end

a.Castable["Ice Block"] = function()
	if not s.MeleeDistance() then
		return false
	end
	wipe(z)
	z.SpellID = 45438--[[Ice Block]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Mana Shield"] = function()
	wipe(z)
	z.SpellID = 1463--[[Mana Shield]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Mage ward"] = function()
	wipe(z)
	z.SpellID = 543--[[Mage ward]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	return s.CheckIfSpellCastable(z)
end

a.Castable["Time Warp"] = function()
	wipe(z)
	z.SpellID = 80353--[[Time Warp]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	return s.CheckIfSpellCastable(z)
end

a.Castable["Evocation"] = function()
	wipe(z)
	z.SpellID = 12051--[[Evocation]]
	z.Buff = z.SpellID
	z.BuffUnit = "player"
	return s.CheckIfSpellCastable(z)
end


a.Castable["Mirror Image"] = function()
	wipe(z)
	z.SpellID = 55342--[[Mirror Image]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Invisibility"] = function()
	wipe(z)
	z.SpellID = 66--[[Invisibility]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Spellsteal"] = function()
	if not s.Casting(nil,"target",1) then
		return false
	end
	wipe(z)
	z.SpellID = 30449--[[Spellsteal]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Counterspell"] = function()
	wipe(z)
	z.SpellID = 2139--[[Counterspell]]
	z.EnemyTargetNeeded = 1
	z.Interrupt = 1
	return s.CheckIfSpellCastable(z)
end


--Frost Functions

a.Castable["Frostbolt"] = function()
	wipe(z)
	z.SpellID = 116--[[Frostbolt]]
	z.MyDebuff = z.SpellID
	z.NotWhileMoving = 1
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Cone of Cold"] = function()
	wipe(z)
	z.SpellID = 120--[[Cone of Cold]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

a.Castable["Frost Nova"] = function()
	wipe(z)
	z.SpellID = 122--[[Frost Nova]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

a.Castable["Frostfire Orb"] = function()
	wipe(z)
	z.SpellID = 92283--[[Frostfire Orb]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Summon Water Elemental"] = function()
	wipe(z)
	z.SpellID = 31687--[[Summon Water Elemental]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Freeze"] = function()
	wipe(z)
	z.SpellID = 33395--[[Freeze]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Deep Freeze"] = function()
	wipe(z)
	z.SpellID = 71757--[[Deep Freeze]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Frostfire Bolt"] = function()
	wipe(z)
	z.SpellID = 44614--[[Frostfire Bolt]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Ice Lance"] = function()
	wipe(z)
	z.SpellID = 30455--[[Ice Lance]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Deep Freeze"] = function()
	wipe(z)
	z.SpellID = 71757--[[Deep Freeze]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end


--Fire Functions
a.Castable["Fireball"] = function()
	wipe(z)
	z.SpellID = 133--[[Fireball]]
	z.NotWhileMoving = 1
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

local PyroBlaster = {
	11366--[[Pyroblast]],
	92315--[[Pyroblast!]],
}

a.Castable["Pyroblast"] = function()
	wipe(z)
	z.SpellID = 11366--[[Pyroblast]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["FOrb"] = function()
	wipe(z)
	z.SpellID = 82731--[[Flame Orb]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Combustion"] = function()
	wipe(z)
	z.SpellID = 11129--[[Combustion]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Living Bomb"] = function()
	wipe(z)
	z.SpellID = 44457--[[Living Bomb]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	z.DebuffSlotNeeded = 1
	return s.CheckIfSpellCastable(z)
end
local CriticalMassDebuffs = {
	17800--[[Shadow and Flame]],
	11095--[[Critical Mass]],
}
a.Castable["Scorch"] = function()
	wipe(z)
	z.SpellID = 2948--[[Scorch]]
	z.Debuff = CriticalMassDebuffs
	z.DebuffSlotNeeded = 1
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Scorch2"] = function()
	wipe(z)
	z.SpellID = 2948--[[Scorch]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

--Arcane Functions

a.Castable["Arcane Blast"] = function()
	wipe(z)
	z.SpellID = 30451--[[Arcane Blast]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Arcane Power"] = function()
	wipe(z)
	z.SpellID = 12042--[[Arcane Power]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Mana Gem"] = function()
	wipe(z)
	z.SpellID = 36799--[[Mana Gem]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Clearcasting"] = function()
	wipe(z)
	z.SpellID = 12536--[[Clearcasting]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Arcane Missiles"] = function()
	wipe(z)
	z.SpellID = 5143--[[Arcane Missiles]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Arcane Barrage"] = function()
	wipe(z)
	z.SpellID = 44425--[[Arcane Barrage]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end
a.Castable["Presence of Mind"] = function()
	wipe(z)
	z.SpellID = 12043--[[Presence of Mind]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

