local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local AddonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local x = SpellFlashAddon.UpdatedVariables

local function SetColor(UseColor, Color, ElseColor)
	if UseColor then
		return Color
	end
	return ElseColor or "Pink"
end

--[[
	White - Default
	Yellow - Limited Time or No Global Cooldown
	Purple - AOE or Positional Damage
	Blue - AOE Debuff
	Orange - Finishing Move
	Aqua - Spell Interrupt, Reflect or Dispel
	Green - Self Buff or Turn Autocast On
	Red - Emergency Mitigation Cooldowns or Turn Autocast Off
	Pink - Optional
]]

-- SpellFlash API: http://wow.curseforge.com/addons/spellflash/pages/api/
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff


local function Buffs()
	if a.GetConfig("Disable_Buff") then return end
	if not IsMounted() and ( not IsResting() or s.InCombat() or x.Enemy  ) then
		if s.TalentMastery(1) then
			if s.Flashable(30482--[[Molten Armor]]) and a.Castable["Molten Armor"]() then
				s.Flash(30482--[[Molten Armor]], "Green")
			end
			if s.Flashable(7302--[[Frost Armor]]) and a.Castable["Frost Armor"]() then
				s.Flash(7302--[[Frost Armor]], "Green")
			end
			if s.Flashable(6117--[[Mage Armor]]) and a.Castable["Mage Armor"]() then
				s.Flash(6117--[[Mage Armor]], "Green")
			end
			if s.Flashable(61316--[[Dalaran Brilliance]]) and a.Castable["Dalaran Brilliance"]() then
				s.Flash(61316--[[Dalaran Brilliance]], "Green")
			elseif s.Flashable(1459--[[Arcane Brilliance]]) and a.Castable["Arcane Brilliance"]() then
				s.Flash(1459--[[Arcane Brilliance]], "Green")
			end
		end
		if s.TalentMastery(2) then
			if s.Flashable(30482--[[Molten Armor]]) and s.PowerPercent("player") > 15 and a.Castable["Molten Armor DPS"]() then
				s.Flash(30482--[[Molten Armor]], "Yellow")
			end
			if s.Flashable(7302--[[Frost Armor]]) and a.Castable["Frost Armor"]() then
				s.Flash(7302--[[Frost Armor]], "Green")
			end
			if s.Flashable(6117--[[Mage Armor]]) and s.PowerPercent("player") < 10 and a.Castable["Mage Armor Mana"]() then
				s.Flash(6117--[[Mage Armor]], "Yellow")
			end
			if s.Flashable(1459--[[Arcane Brilliance]]) and a.Castable["Arcane Brilliance"]() then
				s.Flash(1459--[[Arcane Brilliance]], "Green")
			end
		end
		if s.TalentMastery(3) then
			if s.Flashable(30482--[[Molten Armor]]) and a.Castable["Molten Armor"]() then
				s.Flash(30482--[[Molten Armor]], "Green")
			end
			if s.Flashable(7302--[[Frost Armor]]) and a.Castable["Frost Armor"]() then
				s.Flash(7302--[[Frost Armor]], "Green")
			end
			if s.Flashable(6117--[[Mage Armor]]) and a.Castable["Mage Armor"]() then
				s.Flash(6117--[[Mage Armor]], "Green")
			end
			if s.Flashable(1459--[[Arcane Brilliance]]) and a.Castable["Arcane Brilliance"]() then
				s.Flash(1459--[[Arcane Brilliance]], "Green")
			end
		end
		
	end
end

local function CrowdControl()
	if a.GetConfig("Disable_CC") then return end
	if s.Flashable(118--[[Polymorph]]) and a.Castable["Polymorph"]() then
		s.Flash(118--[[Polymorph]], SetColor(x.ActiveEnemy, "Purple"))
	end
end

local function Survival()
	if a.GetConfig("Disable_Surv") then return end
		if x.ThreatPercent >= 90 then
			if s.Flashable(66--[[Invisibility]]) and x.NoCC and a.Castable["Invisibility"]() then
				s.Flash(66--[[Invisibility]], SetColor(x.ActiveEnemy, "Purple"))
			end
			if s.Flashable(1463--[[Mana Shield]]) and x.NoCC and a.Castable["Mana Shield"]() then
				s.Flash(1463--[[Mana Shield]], SetColor(x.ActiveEnemy, "Purple"))
			end
			if s.Flashable(45438--[[Ice Block]]) and x.NoCC and a.Castable["Ice Block"]() then
				s.Flash(45438--[[Ice Block]], SetColor(x.ActiveEnemy, "Purple"))
			end
		end
		if s.Flashable(1463--[[Mana Shield]]) and s.HealthPercent("player") < 35 and x.NoCC and a.Castable["Mana Shield"]() then
			s.Flash(1463--[[Mana Shield]], SetColor(x.ActiveEnemy, "Purple"))
		end
		if s.Flashable(45438--[[Ice Block]]) and s.HealthPercent("player") < 35 and x.NoCC and a.Castable["Ice Block"]() then
			s.Flash(45438--[[Ice Block]], SetColor(x.ActiveEnemy, "Purple"))
		end
end

local function ManaRegen()
	if a.GetConfig("Disable_Mana") then return end
		if not s.InCombat() then
			if s.Flashable(759--[[Conjure Mana Gem]]) and a.Castable["Conjure Mana Gem"]() then
				s.Flash(759--[[Conjure Mana Gem]], "Green")
			end
		end
	
	if s.ItemFlashable(36799--[[Mana Gem]]) and x.NoCC and s.PowerPercent("player") < 70 and a.ItemCastable["Mana Gem"]() then
		s.FlashItem(36799--[[Mana Gem]], "Aqua")
	elseif s.Flashable(12051--[[Evocation]]) and x.NoCC and s.PowerPercent("player") < 35 and a.Castable["Evocation"]() then
		s.Flash(12051--[[Evocation]], SetColor(x.ActiveEnemy, "Aqua"))
	end
end

local function Interrupts()
	if a.GetConfig("Disable_Kick") then return end
		if s.Flashable(2139--[[Counterspell]]) and x.NoCC and a.Castable["Counterspell"]() then
			s.Flash(2139--[[Counterspell]], SetColor(x.ActiveEnemy, "Aqua"))
		end
end

local function Hero()
	if not a.GetConfig("Enable_Hero") then return end
	local HeroBuffYou = s.Buff(80353--[[Time Warp]], "player") or s.Buff(32182--[[Heroism]], "player") or s.Buff(2825--[[Bloodlust]], "player")
	local HeroBuffCoolDown = s.SpellCooldown(80353--[[Time Warp]], "player") < 0 or s.SpellCooldown(32182--[[Heroism]], "player") < 0 or s.SpellCooldown(2825--[[Bloodlust]], "player") < 0 
	local HeroDeBuffWatch = s.Debuff(80354--[[Temporal Displacement]], "player") or s.Debuff(32182--[[Heroism]], "player") or s.Debuff(2825--[[Bloodlust]], "player")
	if HeroBuffYou or HeroBuffCoolDown or HeroDeBuffWatch then
		if s.Flashable(55342--[[Mirror Image]]) and s.FlashSizePercent(340) and x.NoCC and a.Castable["Mirror Image"]() then
			s.Flash(55342--[[Mirror Image]], SetColor(x.ActiveEnemy, "Yellow"))
		end
	end
end

local function Hero1()
	if a.GetConfig("Enable_Hero") then return end
	if s.Boss() then
		if s.Flashable(55342--[[Mirror Image]]) and s.FlashSizePercent(340) and x.NoCC and a.Castable["Mirror Image"]() then
			s.Flash(55342--[[Mirror Image]], SetColor(x.ActiveEnemy, "Yellow"))
		end
	end
end


local StealAHeal = {
	--druid
	--priest
	--paladin
	--shaman
}
local StealACD = {

}

local StealABoss = {

}

local StealABuff = {

}


local function Stealable()
	if a.GetConfig("Disable_Steal") then return end
		if s.Buff(nil, nil, 30, true, false) then
			if s.Flashable(30449--[[Spellsteal]]) and x.NoCC and a.Castable["Spellsteal"]() then
				s.Flash(30449--[[Spellsteal]], SetColor(x.ActiveEnemy, "Purple"))
			end
		elseif s.Buff(nil, nil, 30, true, false) then
			if s.Flashable(30449--[[Spellsteal]]) and x.NoCC and a.Castable["Spellsteal"]() then
				s.Flash(30449--[[Spellsteal]], SetColor(x.ActiveEnemy, "Purple"))
			end
		end
end

local function FireSpec()
	if a.GetConfig("Disable_Fire") then return end
		if s.TalentMastery(2) then
		
			if GetUnitSpeed("player") == 0 then
				if s.Flashable(11129--[[Combustion]]) and s.MyDebuff(92315--[[Pyroblast]], nil, 2) and s.MyDebuff(11119--[[Ignite]], nil, 2) and s.MyDebuff(44457--[[Living Bomb]], nil, 2) and x.NoCC and a.Castable["Combustion"]() then
					s.Flash(11129--[[Combustion]], SetColor(x.ActiveEnemy, "Yellow"))
				elseif x.NoCC and s.Flashable(82731--[[Flame Orb]]) and s.SpellCooldown(82731--[[Flame Orb]]) < 1 and s.Enemy(nil) then
					s.Flash(82731--[[Flame Orb]])
				elseif s.Flashable(44457--[[Living Bomb]]) and x.NoCC and not s.MyDebuff(44457--[[Living Bomb]]) and a.Castable["Living Bomb"]() then
					s.Flash(44457--[[Living Bomb]], SetColor(x.ActiveEnemy))
				elseif s.Flashable(11366--[[Pyroblast]]) and x.NoCC and s.Buff(48108--[[Hot Streak]], "Player") and a.Castable["Pyroblast"]() then
					s.Flash(11366--[[Pyroblast]], SetColor(x.ActiveEnemy, "Yellow"))
				elseif s.Flashable(2948--[[Scorch]]) and x.NoCC and a.Castable["Scorch"]() then
					s.Flash(2948--[[Scorch]], SetColor(x.ActiveEnemy, "Yellow"))
				elseif s.Flashable(133--[[Fireball]]) and x.NoCC and a.Castable["Fireball"]() then
					s.Flash(133--[[Fireball]], SetColor(x.ActiveEnemy))
				elseif s.Flashable(2948--[[Scorch]]) and x.NoCC and a.Castable["Scorch"]() then
					s.Flash(2948--[[Scorch]], SetColor(x.ActiveEnemy))
				end
			end
	
			if GetUnitSpeed("player") > 0 then
				if s.Flashable(11129--[[Combustion]]) and s.MyDebuff(92315--[[Pyroblast]], nil, 2) and s.MyDebuff(11119--[[Ignite]], nil, 2) and s.MyDebuff(44457--[[Living Bomb]], nil, 2) and x.NoCC and a.Castable["Combustion"]() then
					s.Flash(11129--[[Combustion]], SetColor(x.ActiveEnemy, "Yellow"))
				elseif x.NoCC and s.Flashable(82731--[[Flame Orb]]) and s.SpellCooldown(82731--[[Flame Orb]]) < 1 and s.Enemy(nil) then
					s.Flash(82731--[[Flame Orb]])
				elseif s.Flashable(44457--[[Living Bomb]]) and x.NoCC and not s.MyDebuff(44457--[[Living Bomb]]) and a.Castable["Living Bomb"]() then
					s.Flash(44457--[[Living Bomb]], SetColor(x.ActiveEnemy))
				elseif s.Flashable(11366--[[Pyroblast]]) and x.NoCC and s.Buff(48108--[[Hot Streak]], "Player") and a.Castable["Pyroblast"]() then
					s.Flash(11366--[[Pyroblast]], SetColor(x.ActiveEnemy, "Yellow"))
				elseif s.Flashable(2948--[[Scorch]]) and x.NoCC and s.HasTalent(86914--[[Firestarter]]) and a.Castable["Scorch2"]() then
					s.Flash(2948--[[Scorch]], SetColor(x.ActiveEnemy, "Yellow"))
				end
			end
		end
end

local function FrostSpec()
	if a.GetConfig("Disable_Frost") then return end
		if s.TalentMastery(3) then
	
		if s.Flashable(122--[[Frost Nova]]) and x.NoCC and a.Castable["Frost Nova"]() then
			s.Flash(122--[[Frost Nova]], SetColor(x.ActiveEnemy))
		elseif s.HasTalent(11190--[[Improved Cone of Cold]]) then
			if s.Flashable(120--[[Cone of Cold]]) and x.NoCC and a.Castable["Cone of Cold"]() then
				s.Flash(120--[[Cone of Cold]], SetColor(x.ActiveEnemy))
			end
		end
		
		if s.Flashable(92283--[[Frostfire Orb]]) and x.NoCC and s.SpellCooldown(92283--[[Frostfire Orb]]) < 1 and s.Enemy(nil) then
			s.Flash(92283--[[Frostfire Orb]], SetColor(x.ActiveEnemy))
		elseif s.Flashable(71757--[[Deep Freeze]]) and not s.SpellCooldown(71757--[[Deep Freeze]]) and s.Buff(44544--[[Fingers of Frost]], "player") and x.NoCC and a.Castable["Deep Freeze"]() then
			s.Flash(71757--[[Deep Freeze]], SetColor(x.ActiveEnemy, "Yellow"))
		elseif s.Flashable(44614--[[Frostfire Bolt]]) and s.Buff(9178--[[Brain Freeze]], "player") and x.NoCC and a.Castable["Frostfire Bolt"]() then
			s.Flash(44614--[[Frostfire Bolt]], SetColor(x.ActiveEnemy, "Yellow"))	
		elseif s.Flashable(30455--[[Ice Lance]]) and s.Buff(44544--[[Fingers of Frost]], "player") and x.NoCC and a.Castable["Ice Lance"]() then
			s.Flash(30455--[[Ice Lance]], SetColor(x.ActiveEnemy, "Yellow"))
		elseif s.Flashable(30455--[[Ice Lance]]) and s.Debuff(71757--[[Deep Freeze]]) or s.Debuff(122--[[Frost Nova]]) or (s.HasTalent(11190--[[Improved Cone of Cold]]) and s.Debuff(120--[[Cone of Cold]])) and x.NoCC and a.Castable["Ice Lance"]() then
			s.Flash(30455--[[Ice Lance]], SetColor(x.ActiveEnemy, "Yellow"))	
		elseif s.Flashable(116--[[Frostbolt]]) and x.NoCC and a.Castable["Frostbolt"]() then
			s.Flash(116--[[Frostbolt]], SetColor(x.ActiveEnemy))
		end
	end
end

local function ArcaneSpec()
	if a.GetConfig("Disable_Arcane") then return end
		if s.TalentMastery(1) then
		
		if s.Flashable(6117--[[Mage Armor]]) and a.Castable["Mage Armor"]() then
			s.Flash(6117--[[Mage Armor]], "Yellow")
		end
		
		if s.Flashable(30451--[[Arcane Blast]]) and s.MyBuffStack(30451--[[Arcane Blast]], "player") <= 2 and a.Castable["Arcane Blast"]() then
			s.Flash(30451--[[Arcane Blast]], SetColor(x.ActiveEnemy, "White"))
		elseif a.Castable["Arcane Power"]() then
			if s.Flashable(30451--[[Arcane Blast]]) and s.MyBuffStack(30451--[[Arcane Blast]], "player") >= 3 and a.Castable["Arcane Blast"]() then
				s.Flash(30451--[[Arcane Blast]], SetColor(x.ActiveEnemy, "White"))
			elseif s.Flashable(12042--[[Arcane Power]]) and s.MyBuffStack(30451--[[Arcane Blast]], "player") >= 3 and a.Castable["Arcane Power"]() then
				s.Flash(12042--[[Arcane Power]], SetColor(x.ActiveEnemy, "Yellow"))
			end
		elseif s.Buff(12042--[[Arcane Power]], "player") then
			if s.Flashable(36799--[[Mana Gem]]) and a.Castable["Mana Gem"]() then
				s.Flash(36799--[[Mana Gem]], SetColor(x.ActiveEnemy, "Yellow"))
			elseif s.Flashable(30451--[[Arcane Blast]]) and a.Castable["Arcane Blast"]() then
				s.Flash(30451--[[Arcane Blast]], SetColor(x.ActiveEnemy, "Yellow"))
			end
		end
		
		if s.Flashable(12051--[[Evocation]]) and x.NoCC and s.PowerPercent("player") < 40 and not s.Buff(12042--[[Arcane Power]], "player") and a.Castable["Evocation"]() then
			s.Flash(12051--[[Evocation]], SetColor(x.ActiveEnemy, "Yellow"))
		end
		
		if not a.Castable["Arcane Power"]() and s.Buff(12536--[[Clearcasting]], "player") then
			if s.Flashable(5143--[[Arcane Missiles]]) and a.Castable["Arcane Missiles"]() then
				s.Flash(5143--[[Arcane Missiles]], SetColor(x.ActiveEnemy, "Yellow"))
			elseif s.Flashable(30451--[[Arcane Blast]]) and a.Castable["Arcane Blast"]() then
				s.Flash(30451--[[Arcane Blast]], SetColor(x.ActiveEnemy, "Yellow"))
			end
		end
		
		if not a.Castable["Arcane Power"]() then
			if s.Flashable(30451--[[Arcane Blast]]) and s.MyBuffStack(30451--[[Arcane Blast]], "player") >= 2 and a.Castable["Arcane Blast"]() then
				s.Flash(30451--[[Arcane Blast]], SetColor(x.ActiveEnemy, "White"))
			elseif s.Flashable(44425--[[Arcane Barrage]]) and s.MyBuffStack(30451--[[Arcane Blast]], "player") >= 2 and a.Castable["Arcane Barrage"]() then
				s.Flash(44425--[[Arcane Barrage]], SetColor(x.ActiveEnemy, "White"))
			end
		end
		
	end
end


-- Use this single spam function and remove the multiple spam table below, or just use the multiple spam table below and leave this function in place.
s.Spam[AddonName] = function() if a.GetConfig("spell_flashing_off") then return end a.RunSpamTable()
	
	FireSpec()
	Buffs()
	ArcaneSpec()
	FrostSpec()
	Stealable()
	Interrupts()
	CrowdControl()
	ManaRegen()
	Survival()
	Hero()
	Hero1()
	
end

