local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local AddonName, a = ...
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")
local L = a.Localize
local s = SpellFlashAddon
local x = SpellFlashAddon.UpdatedVariables
local z = {}


-- SpellFlash API: http://wow.curseforge.com/addons/spellflash/pages/api/
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff


-- Example function:
a.Castable["English_Spell_Name"] = function()
	wipe(z)
	z.SpellID = GlobalSpellID--[[English_Spell_Name]]   --this is for defining the correct name of the spell
	z.DebuffSlotNeeded = 1   --use this if you want to make sure that a debuff slot is free on the target so that you do not replace other debuffs
	z.Debuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may only be on a target once in total from everyone and has a cooldown shorter than the debuff duration
	z.MyDebuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may be on the target multiple times by many people and has a cooldown shorter than the debuff duration
	z.Buff = z.SpellID   --place buff name or table of names here if the spell has a buff that may only be on yourself once in total from everyone and has a cooldown shorter than the buff duration
	z.MyBuff = z.SpellID   --place buff name or table of names here if the spell has a buff that may be on the target multiple times by many people and has a cooldown shorter than the buff duration
	z.BuffUnit = "player"   --you will need to use this if the buff or debuff is not applied to your target
	z.Stack = 2   --use this if the buff or debuff requires more than one application
	z.EarlyRefresh = 5   --use this to indicate the buff or debuff early for a desired number of seconds
	z.CastTime = s.CastTime(z.SpellID)   --use this only if you want to replace the spell cast time used for early indication of a buff or debuff
	z.EnemyTargetNeeded = 1   --use this if the spell should have an enemy targeted
	z.TargetThatUsesManaNeeded = 1   --use this if the spell is only useful on a target that uses mana such as mana draining spells
	z.NoRangeCheck = 1   --use this to disable range detection if the spell has a limited range but the detection in this function is not working correctly for the particular spell
	z.NotIfActive = 1   --use this if the spell may be toggled on such as auto attack or on next swing spells, or to disable when casting or channeling, or if the spell has a cooldown
	z.NotWhileMoving = 1   --use this if the spell is not able to be cast while moving
	z.NoPowerCheck = 1   --use this to disable the power checking in this function
	z.EvenIfNotUsable = 1   --use this to disable the usability checking in this function and may be used if currently in the process of completing a prerequisite that will make the spell usable
	z.Unit = "target"   --this should not be used unless the spell is not to be cast on the target
	z.Interrupt = 1   --use this if the target should be casting or channeling an interruptible ability
	return s.CheckIfSpellCastable(z)
end

-- Example function:
a.ItemCastable["English_Item_Name"] = function()
	wipe(z)
	z.ItemID = ItemID--[[English_Item_Name]]   --this is for defining the correct name of the item
	z.DebuffSlotNeeded = 1   --use this if you want to make sure that a debuff slot is free on the target so that you do not replace other debuffs
	z.Debuff = GlobalSpellID--[[English_Spell_Name]]   --place debuff name or table of names here if the item has a debuff that may only be on a target once in total from everyone and has a cooldown shorter than the debuff duration
	z.MyDebuff = GlobalSpellID--[[English_Spell_Name]]   --place debuff name or table of names here if the item has a debuff that may be on the target multiple times by many people and has a cooldown shorter than the debuff duration
	z.Buff = GlobalSpellID--[[English_Spell_Name]]   --place buff name or table of names here if the item has a buff that may only be on yourself once in total from everyone and has a cooldown shorter than the buff duration
	z.MyBuff = GlobalSpellID--[[English_Spell_Name]]   --place buff name or table of names here if the item has a buff that may be on the target multiple times by many people and has a cooldown shorter than the buff duration
	z.BuffUnit = "player"   --you will need to use this if the buff or debuff is not applied to your target
	z.Stack = 2   --use this if the buff or debuff requires more than one application
	z.EarlyRefresh = 5   --use this to indicate the buff or debuff early for a desired number of seconds
	z.CastTime = 0   --use this if the item has a cast time for indication of a buff or debuff early
	z.EnemyTargetNeeded = 1   --use this if the item should have an enemy targeted
	z.TargetThatUsesManaNeeded = 1   --use this if the item is only useful on a target that uses mana such as mana draining items
	z.NoRangeCheck = 1   --use this to disable range detection if the item has a limited range but the detection in this function is not working correctly for the particular item
	z.NoEquipCheck = 1   --use this to disable equippable detection if the item can be equipped but does not need to be
	z.NotIfActive = 1   --use this if the item may be toggled on such as auto attack or on next swing spells, or to disable when casting or channeling, or if the item has a cooldown
	z.NotWhileMoving = 1   --use this if the item is not able to be used while moving
	z.NoPowerCheck = 1   --use this to disable the power checking in this function
	z.EvenIfNotUsable = 1   --use this to disable the usability checking in this function and may be used if currently in the process of completing a prerequisite that will make the item usable
	z.Unit = "target"   --this should not be used unless the item is not to be cast on the target
	z.Interrupt = 1   --use this if the target should be casting or channeling an interruptible ability
	return s.CheckIfItemCastable(z)
end

-- Example function:
a.VehicleCastable["English_Spell_Name"] = function()
	wipe(z)
	z.SpellID = GlobalSpellID--[[English_Spell_Name]]   --this is for defining the correct name of the spell
	z.DebuffSlotNeeded = 1   --use this if you want to make sure that a debuff slot is free on the target so that you do not replace other debuffs
	z.Debuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may only be on a target once in total from everyone and has a cooldown shorter than the debuff duration
	z.MyDebuff = z.SpellID   --place debuff name or table of names here if the spell has a debuff that may be on the target multiple times by many people and has a cooldown shorter than the debuff duration
	z.Buff = z.SpellID   --place buff name or table of names here if the spell has a buff that may only be on yourself once in total from everyone and has a cooldown shorter than the buff duration
	z.MyBuff = z.SpellID   --place buff name or table of names here if the spell has a buff that may be on the target multiple times by many people and has a cooldown shorter than the buff duration
	z.BuffUnit = "vehicle"   --you will need to use this if the buff or debuff is not applied to your target
	z.Stack = 2   --use this if the buff or debuff requires more than one application
	z.EarlyRefresh = 5   --use this to indicate the buff or debuff early for a desired number of seconds
	z.CastTime = 0   --use this if the spell has a cast time for indication of a buff or debuff early
	z.EnemyTargetNeeded = 1   --use this if the spell should have an enemy targeted
	z.TargetThatUsesManaNeeded = 1   --use this if the spell is only useful on a target that uses mana such as mana draining spells
	z.NoRangeCheck = 1   --use this to disable range detection if the spell has a limited range but the detection in this function is not working correctly for the particular spell
	z.NotIfActive = 1   --use this if the spell may be toggled on such as auto attack or on next swing spells, or to disable when casting or channeling, or if the spell has a cooldown
	z.NotWhileMoving = 1   --use this if the spell is not able to be cast while moving
	z.NoPowerCheck = 1   --use this to disable the power checking in this function
	z.EvenIfNotUsable = 1   --use this to disable the usability checking in this function and may be used if currently in the process of completing a prerequisite that will make the spell usable
	z.GlobalVehicleCooldownSpell = GlobalSpellID--[[English_Spell_Name]]   -- use this if spell has more than a global cooldown
	z.Unit = "target"   --this should not be used unless the spell is not to be cast on the target
	z.Interrupt = 1   --use this if the target should be casting or channeling an interruptible ability
	return s.CheckIfVehicleSpellCastable(z)
end



a.Castable["Auto Attack"] = function()
	wipe(z)
	z.SpellID = 6603--[[Auto Attack]]
	z.EnemyTargetNeeded = 1
	z.NotIfActive = 1
	return s.MeleeDistance() and s.CheckIfSpellCastable(z)
end

a.Castable["Shoot"] = function()
	wipe(z)
	z.SpellID = 3018--[[Shoot]]
	z.EnemyTargetNeeded = 1
	z.NotIfActive = 1
	return not s.Shooting() and s.CheckIfSpellCastable(z)
end

--------------Items--------------
a.ItemCastable["Golemblood Potion"] = function()
	wipe(z)
	z.ItemID = 58146--[[Golemblood Potion]]
	z.EnemyTargetNeeded = 1  
	return s.CheckIfItemCastable(z)
end

----------------------Pally BUffs----------------------------

a.Castable["Beacon of light"] = function()
	wipe(z)
	z.SpellID = 53563--[[Beacon of light]]
	z.MyBuff = z.SpellID
	z.BuffUnit  = "raid|range"
	z.EarlyRefresh = 5
	z.NoRangeCheck = 1
	return not IsMounted() and ( not IsResting() or s.InCombat() or s.Enemy("target") or s.Enemy("focus") ) and s.CheckIfSpellCastable(z)
end

a.Castable["Blessing of Kings"] = function()
	wipe(z)
	z.SpellID = 20217--[[Blessing of Kings]]
	z.MyBuff = {
		19740--[[Blessing of Might]],
	}
	z.BuffUnit  = "player"
	z.EarlyRefresh = 60
	z.NoRangeCheck = 1
	return not s.Buff({20217--[[Blessing of Kings]],1126--[[Mark of the wild]]}, "raid|all|range") and not IsMounted() and ( not IsResting() or s.InCombat() or s.Enemy("target") or s.Enemy("focus") ) and s.CheckIfSpellCastable(z)
end

a.Castable["Blessing of Might"] = function()
	wipe(z)
	z.SpellID = 19740--[[Blessing of Might]]
	z.MyBuff = {
		20217--[[Blessing of Kings]],
	}
	z.BuffUnit  = "player"
	z.EarlyRefresh = 60
	z.NoRangeCheck = 1
	return not s.Buff({19740--[[Blessing of Might]],5675--[[Mana Spring Totem]],54424--[[Fel Intelligence]],53138--[[Abomination's Might]],19506--[[Trueshot Aura]],30808--[[Unleashed Rage]]}, "raid|all|range") and not IsMounted() and ( not IsResting() or s.InCombat() or s.Enemy("target") or s.Enemy("focus") ) and s.CheckIfSpellCastable(z)
end

a.Castable["Seal of Insight"] = function()
	wipe(z)
	z.SpellID = 20165--[[Seal of Insight]]
	z.MyBuff = {
		20165--[[Seal of Insight]],
		20164--[[Seal of Justice]],
		20154--[[Seal of Righteousness]],
		31801--[[Seal of Truth]],
	}
	z.EarlyRefresh = 60
	z.BuffUnit  = "player"
	return not IsMounted() and ( not IsResting() or s.Enemy("target") or s.Enemy("focus") ) and not s.InCombat("player") and s.CheckIfSpellCastable(z)
end

a.Castable["Seal of Justice"] = function()
	wipe(z)
	z.SpellID = 20164--[[Seal of Justice]]
	z.MyBuff = {
		20164--[[Seal of Justice]],
		20165--[[Seal of Insight]],
		20154--[[Seal of Righteousness]],
		31801--[[Seal of Truth]],
	}
	z.EarlyRefresh = 60
	z.BuffUnit  = "player"
	return not IsMounted() and ( not IsResting() or s.Enemy("target") or s.Enemy("focus") ) and not s.InCombat("player") and s.CheckIfSpellCastable(z)
end

a.Castable["Seal of Righteousness"] = function()
	wipe(z)
	z.SpellID = 20154--[[Seal of Righteousness]]
	z.MyBuff = {
		20154--[[Seal of Righteousness]],
		20164--[[Seal of Justice]],
		20165--[[Seal of Insight]],
		31801--[[Seal of Truth]],
	}
	z.EarlyRefresh = 60
	z.BuffUnit  = "player"
	return not IsMounted() and ( not IsResting() or s.Enemy("target") or s.Enemy("focus") ) and not s.InCombat("player") and s.CheckIfSpellCastable(z)
end

a.Castable["Seal of Truth"] = function()
	wipe(z)
	z.SpellID = 31801--[[Seal of Truth]]
	z.MyBuff = {
		20154--[[Seal of Righteousness]],
		20164--[[Seal of Justice]],
		20165--[[Seal of Insight]],
		31801--[[Seal of Truth]],
	}
	z.EarlyRefresh = 60
	z.BuffUnit  = "player"
	return not IsMounted() and ( not IsResting() or s.Enemy("target") or s.Enemy("focus") ) and not s.InCombat("player") and s.CheckIfSpellCastable(z)
end

--Seals In Combat
a.Castable["Seal of Insight2"] = function()
	wipe(z)
	z.SpellID = 20165--[[Seal of Insight]]
	z.MyBuff = {
		20165--[[Seal of Insight]],
		20164--[[Seal of Justice]],
		20154--[[Seal of Righteousness]],
		31801--[[Seal of Truth]],
	}
	z.EarlyRefresh = 30
	z.BuffUnit  = "player"
	return not IsMounted() and ( not IsResting() or s.Enemy("target") or s.Enemy("focus") ) and s.InCombat("player") and s.CheckIfSpellCastable(z)
end

a.Castable["Seal of Justice2"] = function()
	wipe(z)
	z.SpellID = 20164--[[Seal of Justice]]
	z.MyBuff = {
		20164--[[Seal of Justice]],
		20165--[[Seal of Insight]],
		20154--[[Seal of Righteousness]],
		31801--[[Seal of Truth]],
	}
	z.EarlyRefresh = 30
	z.BuffUnit  = "player"
	return not IsMounted() and ( not IsResting() or s.Enemy("target") or s.Enemy("focus") ) and s.InCombat("player") and s.CheckIfSpellCastable(z)
end

a.Castable["Seal of Righteousness2"] = function()
	wipe(z)
	z.SpellID = 20154--[[Seal of Righteousness]]
	z.MyBuff = {
		20154--[[Seal of Righteousness]],
		20164--[[Seal of Justice]],
		20165--[[Seal of Insight]],
		31801--[[Seal of Truth]],
	}
	z.EarlyRefresh = 30
	z.BuffUnit  = "player"
	return not IsMounted() and ( not IsResting() or s.Enemy("target") or s.Enemy("focus") ) and s.InCombat("player") and s.CheckIfSpellCastable(z)
end

a.Castable["Seal of Truth2"] = function()
	wipe(z)
	z.SpellID = 31801--[[Seal of Truth]]
	z.MyBuff = {
		20154--[[Seal of Righteousness]],
		20164--[[Seal of Justice]],
		20165--[[Seal of Insight]],
		31801--[[Seal of Truth]],
	}
	z.EarlyRefresh = 30
	z.BuffUnit  = "player"
	return not IsMounted() and ( not IsResting() or s.Enemy("target") or s.Enemy("focus") ) and s.InCombat("player") and s.CheckIfSpellCastable(z)
end

a.Castable["Crusader Aura"] = function()
	wipe(z)
	z.SpellID = 32223--[[Crusader Aura]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return not s.Form(z.Buff) and not IsMounted() and ( not IsResting() or s.InCombat() or s.Enemy("target") or s.Enemy("focus") ) and s.CheckIfSpellCastable(z)
end

a.Castable["Devotion Aura"] = function()
	wipe(z)
	z.SpellID = 465--[[Devotion Aura]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return not s.Form(z.Buff) and not IsMounted() and ( not IsResting() or s.InCombat() or s.Enemy("target") or s.Enemy("focus") ) and s.CheckIfSpellCastable(z)
end

a.Castable["Retribution Aura"] = function()
	wipe(z)
	z.SpellID = 7294--[[Retribution Aura]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return not s.Form(z.Buff) and not IsMounted() and ( not IsResting() or s.InCombat() or s.Enemy("target") or s.Enemy("focus") ) and s.CheckIfSpellCastable(z)
end

a.Castable["Concentration Aura"] = function()
	wipe(z)
	z.SpellID = 19746--[[Concentration Aura]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return not s.Form(z.Buff) and not IsMounted() and ( not IsResting() or s.InCombat() or s.Enemy("target") or s.Enemy("focus") ) and s.CheckIfSpellCastable(z)
end

a.Castable["Resistance Aura"] = function()
	wipe(z)
	z.SpellID = 19891--[[Resistance Aura]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return not s.Form(z.Buff) and not IsMounted() and ( not IsResting() or s.InCombat() or s.Enemy("target") or s.Enemy("focus") ) and s.CheckIfSpellCastable(z)
end



--Basic
a.Castable["Judgement"] = function()
	wipe(z)
	z.SpellID = 20271--[[Judgement]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Consecration"] = function()
	wipe(z)
	z.SpellID = 26573--[[Consecration]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z) and s.MeleeDistance()
end

a.Castable["Divine Plea"] = function()
	wipe(z)
	z.SpellID = 54428--[[Divine Plea]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return s.CheckIfSpellCastable(z)
end


--Holy


a.Castable["Exorcism"] = function()
	wipe(z)
	z.SpellID = 879--[[Exorcism]]
	z.EnemyTargetNeeded = 1
	z.NotWhileMoving = 1
	return s.CheckIfSpellCastable(z)
end

--Prot
a.Castable["Hammer of Justice"] = function()
	wipe(z)
	z.SpellID = 853--[[Hammer of Justice]]
	z.EnemyTargetNeeded = 1
	z.Interrupt = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Righteous Defense"] = function()
	wipe(z)
	z.SpellID = 31789--[[Righteous Defense]]
	return s.CheckIfSpellCastable(z)
end

--Ret
a.Castable["Crusader Strike"] = function()
	wipe(z)
	z.SpellID = 35395--[[Crusader Strike]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Word of Glory"] = function()
	wipe(z)
	z.SpellID = 85673--[[Word of Glory]]
	return s.CheckIfSpellCastable(z)
end


a.Castable["Hand of Reckoning"] = function()
	wipe(z)
	z.SpellID = 62124--[[Hand of Reckoning]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return s.CheckIfSpellCastable(z)
end


a.Castable["Lay on Hands"] = function()
	wipe(z)
	z.SpellID = 633--[[Lay on Hands]]
	return s.CheckIfSpellCastable(z)
end


a.Castable["Righteous Fury"] = function()
	wipe(z)
	z.SpellID = 25780--[[Righteous Fury]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return s.CheckIfSpellCastable(z)
end


a.Castable["Hand of Protection"] = function()
	wipe(z)
	z.SpellID = 1022--[[Hand of Protection]]
	z.Buff = z.SpellID
	return s.CheckIfSpellCastable(z)
end


a.Castable["Holy Wrath"] = function()
	wipe(z)
	z.SpellID = 2812--[[Holy Wrath]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z) and s.MeleeDistance()
end

a.Castable["Divine Protection"] = function()
	wipe(z)
	z.SpellID = 498--[[Divine Protection]]
	z.EnemyTargetNeeded = 1
	return ( s.InCombat() or s.EnemyTargetingYou() ) and s.HealthPercent("player") <= 35 and s.CheckIfSpellCastable(z)
end


a.Castable["Hammer of Wrath"] = function()
	wipe(z)
	z.SpellID = 24275--[[Hammer of Wrath]]
	z.MyDebuff = z.SpellID
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end


a.Castable["Divine Shield"] = function()
	wipe(z)
	z.SpellID = 642--[[Divine Shield]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return s.CheckIfSpellCastable(z)
end


a.Castable["Hand of Freedom"] = function()
	wipe(z)
	z.SpellID = 1044--[[Hand of Freedom]]
	z.Buff = z.SpellID
	return s.CheckIfSpellCastable(z)
end


a.Castable["Hand of Salvation"] = function()
	wipe(z)
	z.SpellID = 1038--[[Hand of Salvation]]
	z.Buff = z.SpellID
	return s.CheckIfSpellCastable(z)
end


a.Castable["Avenging Wrath"] = function()
	wipe(z)
	z.SpellID = 31884--[[Avenging Wrath]]
	z.Buff = z.SpellID
	return s.CheckIfSpellCastable(z)
end


a.Castable["Hand of Sacrifice"] = function()
	wipe(z)
	z.SpellID = 6940--[[Hand of Sacrifice]]
	z.Buff = z.SpellID
	return s.CheckIfSpellCastable(z)
end


a.Castable["Inquisition"] = function(EarlyRefreshSeconds)
	wipe(z)
	z.SpellID = 84963--[[Inquisition]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	z.EarlyRefresh = EarlyRefreshSeconds or 4
	return s.CheckIfSpellCastable(z)
end


--Talent Rank (Holy, Prot, Ret)
a.Castable["Avenger's Shield"] = function(Interrupt)
	wipe(z)
	z.SpellID = 31935--[[Avenger's Shield]]
	z.EnemyTargetNeeded = 1
	z.Interrupt = Interrupt
	return s.CheckIfSpellCastable(z)
end


a.Castable["Holy Shock"] = function()
	wipe(z)
	z.SpellID = 20473--[[Holy Shock]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end


a.Castable["Templar's Verdict"] = function()
	wipe(z)
	z.SpellID = 85256--[[Templar's Verdict]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Holy Shield"] = function()
	wipe(z)
	z.SpellID = 20925--[[Holy Shield]]
	z.EnemyTargetNeeded = 1
	return ( s.InCombat() or s.EnemyTargetingYou() ) and s.HealthPercent("player") <= 75 and s.CheckIfSpellCastable(z)
end


--Has Talent
a.Castable["Ardent Defender"] = function()
	wipe(z)
	z.SpellID = 31850--[[Ardent Defender]]
	z.EnemyTargetNeeded = 1
	return ( s.InCombat() or s.EnemyTargetingYou() ) and s.HealthPercent("player") <= 35 and s.CheckIfSpellCastable(z)
end

a.Castable["Aura Mastery"] = function()
	wipe(z)
	z.SpellID = 31821--[[Aura Mastery]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return s.CheckIfSpellCastable(z)
end

a.Castable["Divine Favor"] = function()
	wipe(z)
	z.SpellID = 31842--[[Divine Favor]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	return s.CheckIfSpellCastable(z)
end

a.Castable["Divine Guardian"] = function()
	wipe(z)
	z.SpellID = 70940--[[Divine Guardian]]
	z.EnemyTargetNeeded = 1
	return ( s.InCombat() or s.EnemyTargetingYou() ) and s.HealthPercent("player") <= 35 and s.CheckIfSpellCastable(z)
end

a.Castable["Divine Storm"] = function()
	wipe(z)
	z.SpellID = 53385--[[Divine Storm]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Guardian of Ancient Kings"] = function()
	wipe(z)
	z.SpellID = 86150--[[Guardian of Ancient Kings]]
	return s.CheckIfSpellCastable(z)
end


a.Castable["Hammer of the Righteous"] = function()
	wipe(z)
	z.SpellID = 53595--[[Hammer of the Righteous]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end


a.Castable["Light of Dawn"] = function()
	wipe(z)
	z.SpellID = 85222--[[Light of Dawn]]
	return s.CheckIfSpellCastable(z)
end


a.Castable["Rebuke"] = function()
	wipe(z)
	z.SpellID = 96231--[[Rebuke]]
	z.EnemyTargetNeeded = 1
	z.Interrupt = 1
	return s.CheckIfSpellCastable(z)
end

a.Castable["Repentance"] = function()
	wipe(z)
	z.SpellID = 20066--[[Repentance]]
	if UnitExists("focus") and not s.Boss("focus") and ( not s.Debuff(nil, "focus") or s.NoDamageCC("focus") ) then
		z.Unit = "focus"
	end
	--z.NotWhileMoving = 1
	z.EnemyTargetNeeded = 1
	local seconds = s.CastTime(z.SpellID) + s.SpellCooldown(z.SpellID) + x.DoubleLag
	return not s.Boss(z.Unit) and s.CheckIfSpellCastable(z) and ( not s.Debuff(nil, z.Unit) or ( s.NoDamageCC(z.Unit) and not s.ImmunityDebuff(z.Unit, seconds) and not s.BreakOnDamageCC(z.Unit, seconds + 5) ) )
end

a.Castable["Shield of the Righteous"] = function()
	wipe(z)
	z.SpellID = 53600--[[Shield of the Righteous]]
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end


a.Castable["Zealotry"] = function()
	wipe(z)
	z.SpellID = 85696--[[Zealotry]]
	z.Buff = z.SpellID
	z.BuffUnit  = "player"
	z.EnemyTargetNeeded = 1
	return s.CheckIfSpellCastable(z)
end

----Holy

a.Castable["Holy Light"] = function()
	wipe(z)
	z.SpellID = 635--[[Holy Light]]
	z.BuffUnit = "player"
	z.NoRangeCheck = 1
	z.Unit = "target"
	return s.CheckIfSpellCastable(z)
end
a.Castable["Divine Light"] = function()
	wipe(z)
	z.SpellID = 82326--[[Divine Light]]
	z.BuffUnit = "player"
	z.NoRangeCheck = 1
	z.Unit = "target"
	return s.CheckIfSpellCastable(z)
end
a.Castable["Flash of Light"] = function()
	wipe(z)
	z.SpellID = 19750--[[Flash of Light]]
	z.BuffUnit = "player"
	z.NoRangeCheck = 1
	z.Unit = "target"
	return s.CheckIfSpellCastable(z)
end
a.Castable["Holy Shock"] = function()
	wipe(z)
	z.SpellID = 20473--[[Holy Shock]]
	z.BuffUnit = "player"
	z.NoRangeCheck = 1
	z.Unit = "target"
	return s.CheckIfSpellCastable(z)
end








